@@ -1,4 +1,4 @@
-/* $XFree86: xc/programs/Xserver/dix/dixfonts.c,v 3.36 2006/12/25 17:04:59 tsi Exp $ */
+/* $XFree86: xc/programs/Xserver/dix/dixfonts.c,v 3.37 2008/03/18 19:50:45 tsi Exp $ */
 /************************************************************************
 Copyright 1987 by Digital Equipment Corporation, Maynard, Massachusetts.
 
@@ -350,6 +350,12 @@ doOpenFont(ClientPtr client, OFclosurePtr c)
 	err = BadFontName;
 	goto bail;
     }
+    if ((pfont->info.firstCol > pfont->info.lastCol) ||
+	(pfont->info.firstRow > pfont->info.lastRow) ||
+	((pfont->info.lastCol - pfont->info.firstCol) > 255)) {
+	err = AllocError;
+	goto bail;
+    }
     if (!pfont->fpe)
 	pfont->fpe = fpe;
     pfont->refcnt++;
