@@ -1,4 +1,4 @@
-/* $XFree86: xc/programs/Xserver/hw/xfree86/common/xf86Bus.c,v 1.99 2007/04/09 15:37:14 tsi Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/common/xf86Bus.c,v 1.101 2008/04/24 20:54:54 tsi Exp $ */
 /*
  * Copyright (c) 1997-2007 by The XFree86 Project, Inc.
  * All rights reserved.
@@ -1181,6 +1181,10 @@ checkConflict(resRange *rgp, resPtr pRes, int entityIndex,
 {
     memType ret;
 
+    /* A range starting at zero is always a conflict */
+    if (rgp->rBase == 0)
+	return 1;
+
     while(pRes) {
 	if (!needCheck(pRes,rgp->type, entityIndex ,state)) {
 	    pRes = pRes->next;
@@ -1198,7 +1202,7 @@ checkConflict(resRange *rgp, resPtr pRes, int entityIndex,
 		    || (rgp->rEnd != pRes->block_end))
 		    return ret;
 	    }
-    break;
+	    break;
 	case ResSparse:
 	    if ((rgp->rBase & rgp->rMask) != rgp->rBase) {
 		xf86Msg(X_ERROR,"sparse io range (base: 0x%lx  mask: 0x%lx)"
@@ -1348,17 +1352,18 @@ xf86PrintResList(int verb, resPtr list)
 		switch (list->res_type & ResExtMask) {
 		case ResBlock:
 		    xf86ErrorFVerb(verb,
-				   "\t[%d] %d\t%ld\t0x%08lx - 0x%08lx (0x%lx)",
+				   "\t[%3d] %3d %3ld 0x%08lx - 0x%08lx (0x%08lx)",
 				   i, list->entityIndex,
 				   (list->res_type & ResDomain) >> 24,
 				   list->block_begin, list->block_end,
 				   list->block_end - list->block_begin + 1);
 		    break;
 		case ResSparse:
-		    xf86ErrorFVerb(verb, "\t[%d] %d\t%ld\t0x%08lx - 0x%08lx",
+		    xf86ErrorFVerb(verb,
+				   "\t[%3d] %3d %3ld 0x%08lx - 0x%08lx",
 				   i, list->entityIndex,
 				   (list->res_type & ResDomain) >> 24,
-				   list->sparse_base,list->sparse_mask);
+				   list->sparse_base, list->sparse_mask);
 		    break;
 		default:
 		    list = list->next;
@@ -1819,15 +1824,10 @@ xf86RegisterResources(int entityIndex, resList list, unsigned long access)
 	    range.type = (range.type & ~ResAccMask) | (access & ResAccMask);
 	}
 	range.type &= ~ResEstimated;	/* Not allowed for drivers */
-#if !(defined(__alpha__) && defined(linux))
-	/* On Alpha Linux, do not check for conflicts, trust the kernel. */
 	if (checkConflict(&range, Acc, entityIndex, SETUP,TRUE))
 	    res = xf86AddResToList(res,&range,entityIndex);
 	else
-#endif
-	{
 	    Acc = xf86AddResToList(Acc,&range,entityIndex);
-	}
 	list++;
     }
     if (list_f)
