@@ -1,4 +1,4 @@
-/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/ati/r128.h,v 1.29 2007/01/09 17:04:45 tsi Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/ati/r128.h,v 1.30 2008/04/02 20:47:57 tsi Exp $ */
 /*
  * Copyright 1999, 2000 ATI Technologies Inc., Markham, Ontario,
  *                      Precision Insight, Inc., Cedar Park, Texas, and
@@ -134,6 +134,13 @@ typedef struct {
 
 				/* CRTC2 registers */
     CARD32     crtc2_gen_cntl;
+    CARD32     crtc2_h_total_disp;
+    CARD32     crtc2_h_sync_strt_wid;
+    CARD32     crtc2_v_total_disp;
+    CARD32     crtc2_v_sync_strt_wid;
+    CARD32     crtc2_offset;
+    CARD32     crtc2_offset_cntl;
+    CARD32     crtc2_pitch;
 
 				/* Flat panel registers */
     CARD32     fp_crtc_h_total_disp;
@@ -159,13 +166,29 @@ typedef struct {
     CARD32     ppll_div_3;
     CARD32     htotal_cntl;
 
+				/* Computed values for PLL2 */
+    CARD32     dot_clock_freq_2;
+    CARD32     pll_output_freq_2;
+    int        feedback_div_2;
+    int        post_div_2;
+
+				/* PLL2 registers */
+    CARD32     p2pll_ref_div;
+    CARD32     p2pll_div_0;
+    CARD32     htotal_cntl2;
+
 				/* DDA register */
     CARD32     dda_config;
     CARD32     dda_on_off;
 
+				/* DDA2 register */
+    CARD32     dda2_config;
+    CARD32     dda2_on_off;
+
 				/* Pallet */
     Bool       palette_valid;
     CARD32     palette[256];
+    CARD32     palette2[256];
 } R128SaveRec, *R128SavePtr;
 
 typedef struct {
@@ -185,6 +208,16 @@ typedef struct {
     DisplayModePtr     mode;
 } R128FBLayout;
 
+typedef enum
+{
+    MT_NONE,
+    MT_CRT,
+    MT_LCD,
+    MT_DFP,
+    MT_CTV,
+    MT_STV
+} R128MonitorType;
+
 typedef struct {
     EntityInfoPtr     pEnt;
     pciVideoPtr       PciInfo;
@@ -301,7 +334,7 @@ typedef struct {
     Bool              IsPCI;            /* Current card is a PCI card */
     drmSize           pciSize;
     drm_handle_t      pciMemHandle;
-    unsigned char     *PCI;             /* Map */
+    drmAddress        PCI;              /* Map */
 
     Bool              allowPageFlip;    /* Enable 3d page flipping */
     Bool              have3DWindows;    /* Are there any 3d clients? */
@@ -310,7 +343,7 @@ typedef struct {
     drmSize           agpSize;
     drm_handle_t      agpMemHandle;     /* Handle from drmAgpAlloc */
     unsigned long     agpOffset;
-    unsigned char     *AGP;             /* Map */
+    drmAddress        AGP;              /* Map */
     int               agpMode;
 
     Bool              CCEInUse;         /* CCE is currently active */
@@ -324,20 +357,20 @@ typedef struct {
     drm_handle_t      ringHandle;       /* Handle from drmAddMap */
     drmSize           ringMapSize;      /* Size of map */
     int               ringSize;         /* Size of ring (in MB) */
-    unsigned char     *ring;            /* Map */
+    drmAddress        ring;             /* Map */
     int               ringSizeLog2QW;
 
     unsigned long     ringReadOffset;   /* Offset into AGP space */
     drm_handle_t      ringReadPtrHandle; /* Handle from drmAddMap */
     drmSize           ringReadMapSize;  /* Size of map */
-    unsigned char     *ringReadPtr;     /* Map */
+    drmAddress        ringReadPtr;      /* Map */
 
 				/* CCE vertex/indirect buffer data */
     unsigned long     bufStart;        /* Offset into AGP space */
     drm_handle_t      bufHandle;       /* Handle from drmAddMap */
     drmSize           bufMapSize;      /* Size of map */
     int               bufSize;         /* Size of buffers (in MB) */
-    unsigned char     *buf;            /* Map */
+    drmAddress        buf;             /* Map */
     int               bufNumBufs;      /* Number of buffers */
     drmBufMapPtr      buffers;         /* Buffer map */
 
@@ -346,7 +379,7 @@ typedef struct {
     drm_handle_t      agpTexHandle;     /* Handle from drmAddMap */
     drmSize           agpTexMapSize;    /* Size of map */
     int               agpTexSize;       /* Size of AGP tex space (in MB) */
-    unsigned char     *agpTex;          /* Map */
+    drmAddress        agpTex;           /* Map */
     int               log2AGPTexGran;
 
 				/* CCE 2D accleration */
@@ -400,6 +433,16 @@ typedef struct {
     I2CBusPtr         pI2CBus;
     CARD32            DDCReg;
 
+    /****** Added for dualhead support *******************/
+    Bool              HasCRTC2;     /* M3/M4 */
+    Bool              IsSecondary;  /* second Screen */
+    Bool              IsPrimary;    /* primary Screen */
+    Bool              UseCRT;       /* force use CRT port as primary */
+    Bool              SwitchingMode;
+    R128MonitorType   DisplayType;  /* Monitor connected on */
+
+    Bool              UseVGAHW;
+
 } R128InfoRec, *R128InfoPtr;
 
 #define R128WaitForFifo(pScrn, entries)                                      \
