@@ -1,4 +1,4 @@
-/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/nsc/nsc_gx1_driver.c,v 1.17 2006/03/16 16:50:11 dawes Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/nsc/nsc_gx1_driver.c,v 1.20 2007/01/23 18:03:05 tsi Exp $ */
 /*
  * File Contents: This is the main module configures the interfacing 
  *                with the X server. The individual modules will be 
@@ -144,7 +144,6 @@
  * Alan Hourihane <alanh@fairlite.demon.co.uk>
  */
 
-#define DEBUG(x)
 #define GEODE_TRACE 0
 #define HWVGA 0
 
@@ -243,7 +242,7 @@ MemOffset GeodeMemOffset[] = {
 static int MemIndex = 0;
 
 static Bool GX1PreInit(ScrnInfoPtr, int);
-static Bool GX1ScreenInit(int, ScreenPtr, int, char **);
+static Bool GX1ScreenInit(int, ScreenPtr, const int, const char **);
 static Bool GX1EnterVT(int, int);
 static void GX1LeaveVT(int, int);
 static void GX1FreeScreen(int, int);
@@ -1757,7 +1756,8 @@ GX1DPMSSet(ScrnInfoPtr pScreenInfo, int mode, int flags)
 *----------------------------------------------------------------------------
 */
 static Bool
-GX1ScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
+GX1ScreenInit(int scrnIndex, ScreenPtr pScreen,
+	      const int argc, const char **argv)
 {
    ScrnInfoPtr pScreenInfo = xf86Screens[pScreen->myNum];
    GeodePtr pGeode;
