@@ -1,4 +1,4 @@
-/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/sis/sis6326_video.c,v 1.19 2004/02/25 17:45:12 twini Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/sis/sis6326_video.c,v 1.23 2006/01/09 15:00:02 dawes Exp $ */
 /*
  * Xv driver for SiS 5597/5598, 6236 and 530/620.
  *
@@ -43,9 +43,8 @@
 #include "regionstr.h"
 
 #include "xf86xv.h"
-#include "Xv.h"
+#include <X11/extensions/Xv.h>
 #include "xaa.h"
-#include "xaalocal.h"
 #include "dixstruct.h"
 #include "fourcc.h"
 
@@ -528,6 +527,13 @@ SIS6326SetupImageVideo(ScreenPtr pScreen)
     XF86VideoAdaptorPtr adapt;
     SISPortPrivPtr pPriv;
 
+#if XF86_VERSION_CURRENT < XF86_VERSION_NUMERIC(4,1,99,1,0)
+    XAAInfoRecPtr pXAA = pSiS->AccelInfoPtr;
+
+    if (!pXAA || !pXAA->FillSolidRects)
+	return NULL;
+#endif
+
     if(!(adapt = xcalloc(1, sizeof(XF86VideoAdaptorRec) +
                             sizeof(SISPortPrivRec) +
                             sizeof(DevUnion))))
@@ -599,7 +605,7 @@ SIS6326SetupImageVideo(ScreenPtr pScreen)
     return adapt;
 }
 
-#if XF86_VERSION_CURRENT < XF86_VERSION_NUMERIC(4,3,99,0,0)
+#if XF86_VERSION_CURRENT < XF86_VERSION_NUMERIC(4,3,99,3,0)
 static Bool
 RegionsEqual(RegionPtr A, RegionPtr B)
 {
@@ -1268,17 +1274,17 @@ SIS6326StopVideo(ScrnInfoPtr pScrn, pointer data, Bool shutdown)
 
   if(shutdown) {
      if(pPriv->videoStatus & CLIENT_VIDEO_ON) {
-       close_overlay(pSiS, pPriv);
-       pPriv->mustwait = 1;
+        close_overlay(pSiS, pPriv);
+        pPriv->mustwait = 1;
      }
      SIS6326FreeOverlayMemory(pScrn);
      pPriv->videoStatus = 0;
      pSiS->VideoTimerCallback = NULL;
   } else {
      if(pPriv->videoStatus & CLIENT_VIDEO_ON) {
-       pPriv->videoStatus = OFF_TIMER | CLIENT_VIDEO_ON;
-       pPriv->offTime = currentTime.milliseconds + OFF_DELAY;
-       pSiS->VideoTimerCallback = SIS6326VideoTimerCallback;
+        pPriv->videoStatus = OFF_TIMER | CLIENT_VIDEO_ON;
+        pPriv->offTime = currentTime.milliseconds + OFF_DELAY;
+        pSiS->VideoTimerCallback = SIS6326VideoTimerCallback;
      }
   }
 }
@@ -1317,7 +1323,7 @@ SIS6326PutImage(
    pPriv->height = height;
    pPriv->width = width;
 
-   /* TW: Pixel formats:
+   /* Pixel formats:
       1. YU12:  3 planes:       H    V
                Y sample period  1    1   (8 bit per pixel)
 	       V sample period  2    2	 (8 bit per pixel, subsampled)
@@ -1392,7 +1398,7 @@ SIS6326PutImage(
    /* update cliplist */
    if(  pPriv->autopaintColorKey &&
         (pPriv->grabbedByV4L ||
-#if XF86_VERSION_CURRENT < XF86_VERSION_NUMERIC(4,3,99,0,0)
+#if XF86_VERSION_CURRENT < XF86_VERSION_NUMERIC(4,3,99,3,0)
 	 !RegionsEqual(&pPriv->clip, clipBoxes)) ) {
 #else
          !REGION_EQUAL(pScrn->pScreen, &pPriv->clip, clipBoxes)) ) {
@@ -1401,8 +1407,8 @@ SIS6326PutImage(
      if(!pPriv->grabbedByV4L)
      	REGION_COPY(pScrn->pScreen, &pPriv->clip, clipBoxes);
      /* draw these */
-#if XF86_VERSION_CURRENT < XF86_VERSION_NUMERIC(4,2,99,0,0)
-     XAAFillSolidRects(pScrn, pPriv->colorKey, GXcopy, ~0,
+#if XF86_VERSION_CURRENT < XF86_VERSION_NUMERIC(4,1,99,1,0)
+     (*pSiS->AccelInfoPtr->FillSolidRects)(pScrn, pPriv->colorKey, GXcopy, ~0,
                     REGION_NUM_RECTS(clipBoxes),
                     REGION_RECTS(clipBoxes));
 #else
@@ -1490,33 +1496,32 @@ SIS6326VideoTimerCallback (ScrnInfoPtr pScrn, Time now)
 
     if(pSiS->adaptor) {
     	pPriv = GET_PORT_PRIVATE(pScrn);
-	if(!pPriv->videoStatus)
-	   pPriv = NULL;
+	if(!pPriv->videoStatus) pPriv = NULL;
     }
 
     if(pPriv) {
-      if(pPriv->videoStatus & TIMER_MASK) {
-        UpdateCurrentTime();
-	if(pPriv->offTime < currentTime.milliseconds) {
+       if(pPriv->videoStatus & TIMER_MASK) {
           if(pPriv->videoStatus & OFF_TIMER) {
-              /* Turn off the overlay */
-	      sridx = inSISREG(SISSR); cridx = inSISREG(SISCR);
-              close_overlay(pSiS, pPriv);
-	      outSISREG(SISSR, sridx); outSISREG(SISCR, cridx);
-	      pPriv->mustwait = 1;
-              pPriv->videoStatus = FREE_TIMER;
-              pPriv->freeTime = currentTime.milliseconds + FREE_DELAY;
-	      pSiS->VideoTimerCallback = SIS6326VideoTimerCallback;
+	     if(pPriv->offTime < now) {
+                /* Turn off the overlay */
+	        sridx = inSISREG(SISSR); cridx = inSISREG(SISCR);
+                close_overlay(pSiS, pPriv);
+	        outSISREG(SISSR, sridx); outSISREG(SISCR, cridx);
+	        pPriv->mustwait = 1;
+                pPriv->videoStatus = FREE_TIMER;
+                pPriv->freeTime = now + FREE_DELAY;
+	        pSiS->VideoTimerCallback = SIS6326VideoTimerCallback;
+	     }
+          } else if(pPriv->videoStatus & FREE_TIMER) {  
+             if(pPriv->freeTime < now) {
+                SIS6326FreeOverlayMemory(pScrn);
+	        pPriv->mustwait = 1;
+                pPriv->videoStatus = 0;
+             }
           } else
-	  if(pPriv->videoStatus & FREE_TIMER) {  
-              SIS6326FreeOverlayMemory(pScrn);
-	      pPriv->mustwait = 1;
-              pPriv->videoStatus = 0;
-          }
-        } else
-	  pSiS->VideoTimerCallback = SIS6326VideoTimerCallback;
-      }
-   }
+	     pSiS->VideoTimerCallback = SIS6326VideoTimerCallback;
+       }
+    }
 }
 
 /* Offscreen surface stuff for v4l */
@@ -1661,8 +1666,8 @@ SIS6326DisplaySurface (
    SIS6326DisplayVideo(pScrn, pPriv);
 
    if(pPriv->autopaintColorKey) {
-#if XF86_VERSION_CURRENT < XF86_VERSION_NUMERIC(4,2,99,0,0)
-   	XAAFillSolidRects(pScrn, pPriv->colorKey, GXcopy, ~0,
+#if XF86_VERSION_CURRENT < XF86_VERSION_NUMERIC(4,1,99,1,0)
+   	(*XAAPTR(pScrn)->FillSolidRects)(pScrn, pPriv->colorKey, GXcopy, ~0,
                     REGION_NUM_RECTS(clipBoxes),
                     REGION_RECTS(clipBoxes));
 #else
