@@ -20,7 +20,7 @@
  * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
  * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  */
-/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/sunleo/leo_driver.c,v 1.12 2005/02/18 02:55:10 dawes Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/sunleo/leo_driver.c,v 1.16 2007/03/15 02:11:37 tsi Exp $ */
 
 #include "xf86.h"
 #include "xf86_OSproc.h"
@@ -40,8 +40,8 @@ static const OptionInfoRec * LeoAvailableOptions(int chipid, int busid);
 static void	LeoIdentify(int flags);
 static Bool	LeoProbe(DriverPtr drv, int flags);
 static Bool	LeoPreInit(ScrnInfoPtr pScrn, int flags);
-static Bool	LeoScreenInit(int Index, ScreenPtr pScreen, int argc,
-			      char **argv);
+static Bool	LeoScreenInit(int Index, ScreenPtr pScreen,
+			      const int argc, const char **argv);
 static Bool	LeoEnterVT(int scrnIndex, int flags);
 static void	LeoLeaveVT(int scrnIndex, int flags);
 static Bool	LeoCloseScreen(int scrnIndex, ScreenPtr pScreen);
@@ -116,7 +116,7 @@ static XF86ModuleVersionInfo sunleoVersRec =
 XF86ModuleData sunleoModuleData = { &sunleoVersRec, leoSetup, NULL };
 
 static pointer
-leoSetup(pointer module, pointer opts, int *errmaj, int *errmin)
+leoSetup(ModuleDescPtr module, pointer opts, int *errmaj, int *errmin)
 {
     static Bool setupDone = FALSE;
 
@@ -449,7 +449,8 @@ LeoPreInit(ScrnInfoPtr pScrn, int flags)
 /* This gets called at the start of each server generation */
 
 static Bool
-LeoScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
+LeoScreenInit(int scrnIndex, ScreenPtr pScreen,
+	      const int argc, const char **argv)
 {
     ScrnInfoPtr pScrn;
     LeoPtr pLeo;
@@ -698,9 +699,9 @@ LeoValidMode(int scrnIndex, DisplayModePtr mode, Bool verbose, int flags)
 /* Mandatory */
 static Bool
 LeoSaveScreen(ScreenPtr pScreen, int mode)
-    /* this function should blank the screen when unblank is FALSE and
-       unblank it when unblank is TRUE -- it doesn't actually seem to be
-       used for much though */
 {
-    return TRUE;
+    ScrnInfoPtr pScrn = xf86Screens[pScreen->myNum];
+    LeoPtr pLeo = GET_LEO_FROM_SCRN(pScrn);
+
+    return xf86SbusSaveScreen(pLeo->psdp, mode);
 }
