@@ -1,3 +1,4 @@
+/* $XFree86: xc/programs/Xserver/hw/xfree86/etc/scanpci.c,v 3.102 2008/06/23 14:42:22 tsi Exp $ */
 /*
  * Copyright 2000 by Egbert Eich
  * Copyright 1995 by Robin Cutshaw <robin@XFree86.Org>
@@ -23,7 +24,6 @@
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
  */
-/* $XFree86: xc/programs/Xserver/hw/xfree86/etc/scanpci.c,v 3.94 2005/10/14 15:16:51 tsi Exp $ */
 
 #include <X11/X.h>
 #include "os.h"
@@ -54,17 +54,16 @@ pciVideoPtr *xf86PciVideoInfo = NULL;
 
 static void usage(void);
 static void identify_card(pciConfigPtr pcr, int verbose);
-static void print_default_class(pciConfigPtr pcr);
-static void print_bridge_pci_class(pciConfigPtr pcr);
-static void print_mach64(pciConfigPtr pcr);
-static void print_i128(pciConfigPtr pcr);
-static void print_dc21050(pciConfigPtr pcr);
+static void dump_config(pciConfigPtr pcr, int verbose);
+static void print_header_type_0(pciConfigPtr pcr);
+static void print_header_type_1(pciConfigPtr pcr);
+static void print_header_type_2(pciConfigPtr pcr);
 static void print_simba(pciConfigPtr pcr);
 static void print_460gx_sac(pciConfigPtr pcr);
 static void print_460gx_pxb(pciConfigPtr pcr);
 static void print_460gx_gxb(pciConfigPtr pcr);
 
-#define MAX_DEV_PER_VENDOR 40
+#define MAX_DEV_PER_VENDOR 8
 
 typedef struct {
     unsigned int Vendor;
@@ -75,51 +74,6 @@ typedef struct {
 } pciVendorDevFuncInfo;
 
 static pciVendorDevFuncInfo vendorDeviceFuncInfo[] = {
-    { PCI_VENDOR_ATI, {
-	{ PCI_CHIP_MACH64CT, print_mach64 },
-	{ PCI_CHIP_MACH64CX, print_mach64 },
-	{ PCI_CHIP_MACH64ET, print_mach64 },
-	{ PCI_CHIP_MACH64GB, print_mach64 },
-	{ PCI_CHIP_MACH64GD, print_mach64 },
-	{ PCI_CHIP_MACH64GI, print_mach64 },
-	{ PCI_CHIP_MACH64GL, print_mach64 },
-	{ PCI_CHIP_MACH64GM, print_mach64 },
-	{ PCI_CHIP_MACH64GN, print_mach64 },
-	{ PCI_CHIP_MACH64GO, print_mach64 },
-	{ PCI_CHIP_MACH64GP, print_mach64 },
-	{ PCI_CHIP_MACH64GQ, print_mach64 },
-	{ PCI_CHIP_MACH64GR, print_mach64 },
-	{ PCI_CHIP_MACH64GS, print_mach64 },
-	{ PCI_CHIP_MACH64GT, print_mach64 },
-	{ PCI_CHIP_MACH64GU, print_mach64 },
-	{ PCI_CHIP_MACH64GV, print_mach64 },
-	{ PCI_CHIP_MACH64GW, print_mach64 },
-	{ PCI_CHIP_MACH64GX, print_mach64 },
-	{ PCI_CHIP_MACH64GY, print_mach64 },
-	{ PCI_CHIP_MACH64GZ, print_mach64 },
-	{ PCI_CHIP_MACH64LB, print_mach64 },
-	{ PCI_CHIP_MACH64LD, print_mach64 },
-	{ PCI_CHIP_MACH64LG, print_mach64 },
-	{ PCI_CHIP_MACH64LI, print_mach64 },
-	{ PCI_CHIP_MACH64LM, print_mach64 },
-	{ PCI_CHIP_MACH64LN, print_mach64 },
-	{ PCI_CHIP_MACH64LP, print_mach64 },
-	{ PCI_CHIP_MACH64LQ, print_mach64 },
-	{ PCI_CHIP_MACH64LR, print_mach64 },
-	{ PCI_CHIP_MACH64LS, print_mach64 },
-	{ PCI_CHIP_MACH64VT, print_mach64 },
-	{ PCI_CHIP_MACH64VU, print_mach64 },
-	{ PCI_CHIP_MACH64VV, print_mach64 },
-	{ 0x0000,  NULL } } },
-    { PCI_VENDOR_DIGITAL, {
-	{ PCI_CHIP_DC21050, print_dc21050},
-	{ 0x0000, NULL } } },
-    { PCI_VENDOR_NUMNINE, {
-	{ PCI_CHIP_I128, print_i128 },
-	{ PCI_CHIP_I128_2, print_i128 },
-	{ PCI_CHIP_I128_T2R, print_i128 },
-	{ PCI_CHIP_I128_T2R4, print_i128 },
-	{ 0x0000, NULL } } },
     { PCI_VENDOR_SUN, {
 	{ PCI_CHIP_SIMBA, print_simba },
 	{ 0x0000, NULL } } },
@@ -137,7 +91,8 @@ static void
 usage(void)
 {
     printf("Usage: scanpci [-v12OfV]\n");
-    printf("           -v print config space\n");
+    printf("           -v print config space verbosely\n");
+    printf("           -x print config space in hexadecimal\n");
     printf("           -1 config type 1\n");
     printf("           -2 config type 2\n");
     printf("           -O use OS config support\n");
@@ -149,14 +104,11 @@ int
 main(int argc, char *argv[])
 {
     pciConfigPtr pPCI, *pcrpp = NULL;
-    int Verbose = 0;
-    int i = 0;
-    int force = 0;
-    int c;
+    int Verbose = 0, force = 0, dump = 0, i = 0, c;
 
     xf86Info.pciFlags = PCIProbe1;
 
-    while ((c = getopt(argc, argv, "?v12OfV:")) != -1)
+    while ((c = getopt(argc, argv, "?v12OfdxaV:")) != -1)
 	switch(c) {
 	case 'v':
 	    Verbose++;
@@ -176,6 +128,13 @@ main(int argc, char *argv[])
 	case 'V':
 	    xf86Verbose = atoi(optarg);
 	    break;
+	case 'd':
+	case 'x':
+	    dump++;
+	    break;
+	case 'a':	/* Not documented */
+	    xf86Info.estimateSizesAggressively++;
+	    break;
 	case '?':
 	default:
 	    usage();
@@ -204,8 +163,11 @@ main(int argc, char *argv[])
 	exit (1);
     }
 
-    while ((pPCI = pcrpp[i++]))
+    while ((pPCI = pcrpp[i++])) {
 	identify_card(pPCI, Verbose);
+	if (dump)
+	    dump_config(pPCI, dump);
+    }
 
     if (Verbose > 1) {
 	printf("\nPCI bus linkages:\n\n");
@@ -262,27 +224,19 @@ identify_card(pciConfigPtr pcr, int verbose)
 
     if (!foundit)
 	printf(" Device unknown\n");
-    else {
+    else
 	printf("\n");
-	if (verbose) {
-	    for (i = 0;  vdf[i].Vendor;  i++) {
-		if (vdf[i].Vendor == pcr->pci_vendor) {
-		    for (j = 0;  vdf[i].Device[j].DeviceID;  j++) {
-			if (vdf[i].Device[j].DeviceID == pcr->pci_device) {
-			    (*vdf[i].Device[j].func)(pcr);
-			    return;
-			}
-		    }
-		    break;
-		}
-	    }
-	}
-    }
 
-    if (verbose && !(pcr->pci_header_type & 0x7f) &&
-	(pcr->pci_subsys_vendor != 0 || pcr->pci_subsys_card != 0) &&
-	(pcr->pci_vendor != pcr->pci_subsys_vendor ||
-	 pcr->pci_device != pcr->pci_subsys_card)) {
+    if (verbose <= 0)
+	return;
+
+    if (!(pcr->pci_header_type & 0x7f) &&
+	(((pcr->pci_subsys_vendor != 0) &&
+	  (pcr->pci_subsys_vendor != 0xffffu)) ||
+         ((pcr->pci_subsys_card != 0) &&
+	  (pcr->pci_subsys_card != 0xffffu))) &&
+	((pcr->pci_vendor != pcr->pci_subsys_vendor) ||
+	 (pcr->pci_device != pcr->pci_subsys_card))) {
 	foundit = 0;
 	foundvendor = 0;
 	printf(" CardVendor 0x%04x card 0x%04x",
@@ -306,149 +260,293 @@ identify_card(pciConfigPtr pcr, int verbose)
 	printf("\n");
     }
 
-    if (verbose) {
-	printf("  STATUS    0x%04x  COMMAND 0x%04x\n",
-	       pcr->pci_status, pcr->pci_command);
-	printf("  CLASS     0x%02x 0x%02x 0x%02x  REVISION 0x%02x\n",
-	       pcr->pci_base_class, pcr->pci_sub_class, pcr->pci_prog_if,
-	       pcr->pci_rev_id);
-	if ((pcr->pci_base_class == PCI_CLASS_BRIDGE) &&
-	    (pcr->pci_sub_class == PCI_SUBCLASS_BRIDGE_PCI))
-	    print_bridge_pci_class(pcr);
-	else
-	    print_default_class(pcr);
+    printf("  STATUS    0x%04x  COMMAND 0x%04x\n",
+	   pcr->pci_status, pcr->pci_command);
+    printf("  CLASS     0x%02x 0x%02x 0x%02x  REVISION 0x%02x\n",
+	   pcr->pci_base_class, pcr->pci_sub_class, pcr->pci_prog_if,
+	   pcr->pci_rev_id);
+    printf("  BIST      0x%02x  HEADER 0x%02x  LATENCY 0x%02x  CACHE 0x%02x\n",
+	   pcr->pci_bist, pcr->pci_header_type, pcr->pci_latency_timer,
+	   pcr->pci_cache_line_size);
+
+    for (i = 0;  vdf[i].Vendor;  i++) {
+	if (vdf[i].Vendor != pcr->pci_vendor)
+	    continue;
+
+	for (j = 0;  vdf[i].Device[j].DeviceID;  j++) {
+	    if (vdf[i].Device[j].DeviceID != pcr->pci_device)
+		continue;
+
+	    (*vdf[i].Device[j].func)(pcr);
+	    return;
+	}
+
+	break;
+    }
+
+    switch (pcr->pci_header_type & 0x7f) {
+    case 0:
+	print_header_type_0(pcr);
+	break;
+
+    case 1:
+	print_header_type_1(pcr);
+	break;
+
+    case 2:
+	print_header_type_2(pcr);
+	break;
+
+    default:
+	printf("  Unknown header type 0x%02x\n", pcr->pci_header_type);
+	break;
     }
 }
 
 static void
-print_default_class(pciConfigPtr pcr)
+dump_config(pciConfigPtr pcr, int verbose)
 {
-    printf("  BIST      0x%02x  HEADER 0x%02x  LATENCY 0x%02x  CACHE 0x%02x\n",
-	   pcr->pci_bist, pcr->pci_header_type, pcr->pci_latency_timer,
-	   pcr->pci_cache_line_size);
-    if (pcr->pci_base0) {
+    int i, limit;
+
+    if ((pcr->pci_header_type & 0x7f) == 2)
+	limit = 32;
+    else
+	limit = 16;
+
+    for (i = 0;  i < limit;  i++) {
+	if (!(i & 3))
+	    printf("    0x%03x: ", i * 4);
+	if (!(i & 1))
+	    printf(" ");
+	/* Always display little-endian */
+	printf(" %02x%02x%02x%02x",
+	       (CARD8)(pcr->cfgspc.dwords[i]),
+	       (CARD8)(pcr->cfgspc.dwords[i] >> 8),
+	       (CARD8)(pcr->cfgspc.dwords[i] >> 16),
+	       (CARD8)(pcr->cfgspc.dwords[i] >> 24));
+	if ((i & 3) == 3)
+	    printf("\n");
+    }
+
+    if (verbose <= 1)
+	return;
+
+    i = limit * 4;
+    if (verbose > 2)
+	limit = 4096;
+    else
+	limit = 256;
+
+    /* Print the rest */
+    for (;  i < limit;  i += 4) {
+	CARD32 pcireg = pciReadLong(pcr->tag, i);
+
+	if ((pcireg == (CARD32)(-1L)) && (i == 256))
+	    break;
+	if (!(i & 15))
+	    printf("    0x%03x: ", i);
+	if (!(i & 7))
+	    printf(" ");
+	/* Always display little-endian */
+	printf(" %02x%02x%02x%02x",
+	       (CARD8)(pcireg), (CARD8)(pcireg >> 8),
+	       (CARD8)(pcireg >> 16), (CARD8)(pcireg >> 24));
+	if ((i & 15) == 12)
+	    printf("\n");
+    }
+}
+
+static void
+print_header_type_0(pciConfigPtr pcr)
+{
+    if (pcr->basesize[0]) {
 	if ((pcr->pci_base0 & 0x7) == 0x4) {
-	    printf("  BASE0     0x%08x%08x  addr 0x%08x%08x  MEM%s 64BIT\n",
+	    printf("  BASE0     0x%08x%08x  addr 0x%08x%08x",
 		   (int)pcr->pci_base1, (int)pcr->pci_base0,
 		   (int)pcr->pci_base1,
 		   (int)(pcr->pci_base0 &
-			 (pcr->pci_base0 & 0x1 ? 0xFFFFFFFC : 0xFFFFFFF0)),
-		   (pcr->pci_base0 & 0x8) ? " PREFETCHABLE" :"");
+			 (pcr->pci_base0 & 0x1 ? 0xFFFFFFFC : 0xFFFFFFF0)));
+	    printf("  %ssize ",
+		   (pcr->minBasesize & (2 << 0)) ? "" : "estimated ");
+	    if (pcr->basesize[0] < 32)
+		printf("0x%08x", 1 << pcr->basesize[0]);
+	    else
+		printf("0x%08x00000000", 1 << (pcr->basesize[0] - 32));
+	    printf("  MEM%s 64BIT\n",
+		   (pcr->pci_base0 & 0x8) ? " PREFETCHABLE" : "");
 	} else {
-	    printf("  BASE0     0x%08x  addr 0x%08x  %s%s\n",
+	    printf("  BASE0     0x%08x  addr 0x%08x",
 		   (int)pcr->pci_base0,
 		   (int)(pcr->pci_base0 &
-			 (pcr->pci_base0 & 0x1 ? 0xFFFFFFFC : 0xFFFFFFF0)),
+			 (pcr->pci_base0 & 0x1 ? 0xFFFFFFFC : 0xFFFFFFF0)));
+	    printf("  %ssize 0x%08x",
+		   (pcr->minBasesize & (2 << 0)) ? "" : "estimated ",
+		   1 << pcr->basesize[0]);
+	    printf("  %s%s\n",
 		   (pcr->pci_base0 & 0x1) ? "I/O" : "MEM",
-		   ((pcr->pci_base0 & 0x9) == 0x8) ? " PREFETCHABLE" :"");
+		   ((pcr->pci_base0 & 0x9) == 0x8) ? " PREFETCHABLE" : "");
 	}
     }
-    if ((pcr->pci_base1) && ((pcr->pci_base0 & 0x7) != 0x4)) {
+
+    if (pcr->basesize[1]) {
 	if ((pcr->pci_base1 & 0x7) == 0x4) {
-	    printf("  BASE1     0x%08x%08x  addr 0x%08x%08x  MEM%s 64BIT\n",
+	    printf("  BASE1     0x%08x%08x  addr 0x%08x%08x",
 		   (int)pcr->pci_base2, (int)pcr->pci_base1,
 		   (int)pcr->pci_base2,
 		   (int)(pcr->pci_base1 &
-			 (pcr->pci_base1 & 0x1 ? 0xFFFFFFFC : 0xFFFFFFF0)),
-		   (pcr->pci_base1 & 0x8) ? " PREFETCHABLE" :"");
+			 (pcr->pci_base1 & 0x1 ? 0xFFFFFFFC : 0xFFFFFFF0)));
+	    printf("  %ssize ",
+		   (pcr->minBasesize & (2 << 1)) ? "" : "estimated ");
+	    if (pcr->basesize[1] < 32)
+		printf("0x%08x", 1 << pcr->basesize[1]);
+	    else
+		printf("0x%08x00000000", 1 << (pcr->basesize[1] - 32));
+	    printf("  MEM%s 64BIT\n",
+		   (pcr->pci_base1 & 0x8) ? " PREFETCHABLE" : "");
 	} else {
-	    printf("  BASE1     0x%08x  addr 0x%08x  %s%s\n",
+	    printf("  BASE1     0x%08x  addr 0x%08x",
 		   (int)pcr->pci_base1,
 		   (int)(pcr->pci_base1 &
-			 (pcr->pci_base1 & 0x1 ? 0xFFFFFFFC : 0xFFFFFFF0)),
+			 (pcr->pci_base1 & 0x1 ? 0xFFFFFFFC : 0xFFFFFFF0)));
+	    printf("  %ssize 0x%08x",
+		   (pcr->minBasesize & (2 << 1)) ? "" : "estimated ",
+		   1 << pcr->basesize[1]);
+	    printf("  %s%s\n",
 		   (pcr->pci_base1 & 0x1) ? "I/O" : "MEM",
-		   ((pcr->pci_base1 & 0x9) == 0x8) ? " PREFETCHABLE" :"");
+		   ((pcr->pci_base1 & 0x9) == 0x8) ? " PREFETCHABLE" : "");
 	}
     }
-    if ((pcr->pci_base2) && ((pcr->pci_base1 & 0x7) != 0x4)) {
+
+    if (pcr->basesize[2]) {
 	if ((pcr->pci_base2 & 0x7) == 0x4) {
-	    printf("  BASE2     0x%08x%08x  addr 0x%08x%08x  MEM%s 64BIT\n",
+	    printf("  BASE2     0x%08x%08x  addr 0x%08x%08x",
 		   (int)pcr->pci_base3, (int)pcr->pci_base2,
 		   (int)pcr->pci_base3,
 		   (int)(pcr->pci_base2 &
-			 (pcr->pci_base2 & 0x1 ? 0xFFFFFFFC : 0xFFFFFFF0)),
-		   (pcr->pci_base2 & 0x8) ? " PREFETCHABLE" :"");
+			 (pcr->pci_base2 & 0x1 ? 0xFFFFFFFC : 0xFFFFFFF0)));
+	    printf("  %ssize ",
+		   (pcr->minBasesize & (2 << 2)) ? "" : "estimated ");
+	    if (pcr->basesize[2] < 32)
+		printf("0x%08x", 1 << pcr->basesize[2]);
+	    else
+		printf("0x%08x00000000", 1 << (pcr->basesize[2] - 32));
+	    printf("  MEM%s 64BIT\n",
+		   (pcr->pci_base2 & 0x8) ? " PREFETCHABLE" : "");
 	} else {
-	    printf("  BASE2     0x%08x  addr 0x%08x  %s%s\n",
+	    printf("  BASE2     0x%08x  addr 0x%08x",
 		   (int)pcr->pci_base2,
 		   (int)(pcr->pci_base2 &
-			 (pcr->pci_base2 & 0x1 ? 0xFFFFFFFC : 0xFFFFFFF0)),
+			 (pcr->pci_base2 & 0x1 ? 0xFFFFFFFC : 0xFFFFFFF0)));
+	    printf("  %ssize 0x%08x",
+		   (pcr->minBasesize & (2 << 2)) ? "" : "estimated ",
+		   1 << pcr->basesize[2]);
+	    printf("  %s%s\n",
 		   (pcr->pci_base2 & 0x1) ? "I/O" : "MEM",
-		   ((pcr->pci_base2 & 0x9) == 0x8) ? " PREFETCHABLE" :"");
+		   ((pcr->pci_base2 & 0x9) == 0x8) ? " PREFETCHABLE" : "");
 	}
     }
-    if ((pcr->pci_base3) && ((pcr->pci_base2 & 0x7) != 0x4)) {
+
+    if (pcr->basesize[3]) {
 	if ((pcr->pci_base3 & 0x7) == 0x4) {
-	    printf("  BASE3     0x%08x%08x  addr 0x%08x%08x  MEM%s 64BIT\n",
+	    printf("  BASE3     0x%08x%08x  addr 0x%08x%08x",
 		   (int)pcr->pci_base4, (int)pcr->pci_base3,
 		   (int)pcr->pci_base4,
 		   (int)(pcr->pci_base3 &
-			 (pcr->pci_base3 & 0x1 ? 0xFFFFFFFC : 0xFFFFFFF0)),
-		   (pcr->pci_base3 & 0x8) ? " PREFETCHABLE" :"");
+			 (pcr->pci_base3 & 0x1 ? 0xFFFFFFFC : 0xFFFFFFF0)));
+	    printf("  %ssize ",
+		   (pcr->minBasesize & (2 << 3)) ? "" : "estimated ");
+	    if (pcr->basesize[3] < 32)
+		printf("0x%08x", 1 << pcr->basesize[3]);
+	    else
+		printf("0x%08x00000000", 1 << (pcr->basesize[3] - 32));
+	    printf("  MEM%s 64BIT\n",
+		   (pcr->pci_base3 & 0x8) ? " PREFETCHABLE" : "");
 	} else {
-	    printf("  BASE3     0x%08x  addr 0x%08x  %s%s\n",
+	    printf("  BASE3     0x%08x  addr 0x%08x",
 		   (int)pcr->pci_base3,
 		   (int)(pcr->pci_base3 &
-			 (pcr->pci_base3 & 0x1 ? 0xFFFFFFFC : 0xFFFFFFF0)),
+			 (pcr->pci_base3 & 0x1 ? 0xFFFFFFFC : 0xFFFFFFF0)));
+	    printf("  %ssize 0x%08x",
+		   (pcr->minBasesize & (2 << 3)) ? "" : "estimated ",
+		   1 << pcr->basesize[3]);
+	    printf("  %s%s\n",
 		   (pcr->pci_base3 & 0x1) ? "I/O" : "MEM",
-		   ((pcr->pci_base3 & 0x9) == 0x8) ? " PREFETCHABLE" :"");
+		   ((pcr->pci_base3 & 0x9) == 0x8) ? " PREFETCHABLE" : "");
 	}
     }
-    if ((pcr->pci_base4) && ((pcr->pci_base3 & 0x7) != 0x4)) {
+
+    if (pcr->basesize[4]) {
 	if ((pcr->pci_base4 & 0x7) == 0x4) {
-	    printf("  BASE4     0x%08x%08x  addr 0x%08x%08x  MEM%s 64BIT\n",
+	    printf("  BASE4     0x%08x%08x  addr 0x%08x%08x",
 		   (int)pcr->pci_base5, (int)pcr->pci_base4,
 		   (int)pcr->pci_base5,
 		   (int)(pcr->pci_base4 &
-			 (pcr->pci_base4 & 0x1 ? 0xFFFFFFFC : 0xFFFFFFF0)),
-		   (pcr->pci_base4 & 0x8) ? " PREFETCHABLE" :"");
+			 (pcr->pci_base4 & 0x1 ? 0xFFFFFFFC : 0xFFFFFFF0)));
+	    printf("  %ssize ",
+		   (pcr->minBasesize & (2 << 4)) ? "" : "estimated ");
+	    if (pcr->basesize[4] < 32)
+		printf("0x%08x", 1 << pcr->basesize[4]);
+	    else
+		printf("0x%08x00000000", 1 << (pcr->basesize[4] - 32));
+	    printf("  MEM%s 64BIT\n",
+		   (pcr->pci_base4 & 0x8) ? " PREFETCHABLE" : "");
 	} else {
-	    printf("  BASE4     0x%08x  addr 0x%08x  %s%s\n",
+	    printf("  BASE4     0x%08x  addr 0x%08x",
 		   (int)pcr->pci_base4,
 		   (int)(pcr->pci_base4 &
-			 (pcr->pci_base4 & 0x1 ? 0xFFFFFFFC : 0xFFFFFFF0)),
+			 (pcr->pci_base4 & 0x1 ? 0xFFFFFFFC : 0xFFFFFFF0)));
+	    printf("  %ssize 0x%08x",
+		   (pcr->minBasesize & (2 << 4)) ? "" : "estimated ",
+		   1 << pcr->basesize[4]);
+	    printf("  %s%s\n",
 		   (pcr->pci_base4 & 0x1) ? "I/O" : "MEM",
-		   ((pcr->pci_base4 & 0x9) == 0x8) ? " PREFETCHABLE" :"");
+		   ((pcr->pci_base4 & 0x9) == 0x8) ? " PREFETCHABLE" : "");
 	}
     }
-    if ((pcr->pci_base5) && ((pcr->pci_base4 & 0x7) != 0x4)) {
-	printf("  BASE5     0x%08x  addr 0x%08x  %s%s%s\n",
-	       (int)pcr->pci_base5,
-	       (int)(pcr->pci_base5 &
-		     (pcr->pci_base5 & 0x1 ?  0xFFFFFFFC : 0xFFFFFFF0)),
-	       (pcr->pci_base5 & 0x1) ? "I/O" : "MEM",
-	       ((pcr->pci_base5 & 0x9) == 0x8) ? " PREFETCHABLE" :"",
-	       ((pcr->pci_base5 & 0x7) == 0x4) ? " 64BIT" : "");
+
+    if (pcr->basesize[5]) {
+	if ((pcr->pci_base5 & 0x7) == 0x4) {
+	    printf("  BASE5     0x%08x  (invalid)\n", (int)pcr->pci_base5);
+	} else {
+	    printf("  BASE5     0x%08x  addr 0x%08x",
+		   (int)pcr->pci_base5,
+		   (int)(pcr->pci_base5 &
+			(pcr->pci_base5 & 0x1 ?  0xFFFFFFFC : 0xFFFFFFF0)));
+	    printf("  %ssize 0x%08x",
+		   (pcr->minBasesize & (2 << 5)) ? "" : "estimated ",
+		   1 << pcr->basesize[5]);
+	    printf("  %s%s\n",
+		   (pcr->pci_base5 & 0x1) ? "I/O" : "MEM",
+		   ((pcr->pci_base5 & 0x9) == 0x8) ? " PREFETCHABLE" : "");
+	}
     }
-    if (pcr->pci_baserom)
-	printf("  BASEROM   0x%08x  addr 0x%08x  %sdecode-enabled\n",
-	       (int)pcr->pci_baserom, (int)(pcr->pci_baserom & 0xFFFF8000),
-	       pcr->pci_baserom & 0x1 ? "" : "not-");
+
+    if (pcr->basesize[6]) {
+	printf("  BASEROM   0x%08x  addr 0x%08x",
+	       (int)pcr->pci_baserom, (int)(pcr->pci_baserom & 0xFFFFF800));
+	printf("  %ssize 0x%08x",
+	       (pcr->minBasesize & (2 << 6)) ? "" : "estimated ",
+	       1 << pcr->basesize[6]);
+	printf("  %senabled\n",
+	       pcr->pci_baserom & 0x1 ? "" : "not ");
+    }
+
     if (pcr->pci_max_min_ipin_iline)
 	printf("  MAX_LAT   0x%02x  MIN_GNT 0x%02x"
 	       "  INT_PIN 0x%02x  INT_LINE 0x%02x\n",
 	       pcr->pci_max_lat, pcr->pci_min_gnt,
 	       pcr->pci_int_pin, pcr->pci_int_line);
+
     if (pcr->pci_user_config)
 	printf("  BYTE_0    0x%02x  BYTE_1  0x%02x"
-	       "  BYTE_2  0x%02x  BYTE_3  0x%02x\n",
+	       "  BYTE_2  0x%02x  BYTE_3   0x%02x\n",
 	       (int)pcr->pci_user_config_0, (int)pcr->pci_user_config_1,
 	       (int)pcr->pci_user_config_2, (int)pcr->pci_user_config_3);
 }
 
-#define PCI_B_FAST_B_B 0x80
-#define PCI_B_SB_RESET 0x40
-#define PCI_B_M_ABORT  0x20
-#define PCI_B_VGA_EN   0x08
-#define PCI_B_ISA_EN   0x04
-#define PCI_B_SERR_EN  0x02
-#define PCI_B_P_ERR    0x01
-
 static void
-print_bridge_pci_class(pciConfigPtr pcr)
+print_header_type_1(pciConfigPtr pcr)
 {
-    printf("  HEADER    0x%02x  LATENCY 0x%02x\n",
-	   pcr->pci_header_type, pcr->pci_latency_timer);
     printf("  PRIBUS    0x%02x  SECBUS 0x%02x  SUBBUS 0x%02x\n",
 	   pcr->pci_primary_bus_number, pcr->pci_secondary_bus_number,
 	   pcr->pci_subordinate_bus_number);
@@ -459,17 +557,24 @@ print_bridge_pci_class(pciConfigPtr pcr)
 	pcr->pci_upper_io_base || pcr->pci_upper_io_limit) {
 	if (((pcr->pci_io_base & 0x0f) == 0x01) ||
 	    ((pcr->pci_io_limit & 0x0f) == 0x01)) {
-	    printf("  IOBASE    0x%04x%04x  IOLIM 0x%04x%04x\n",
-		   pcr->pci_upper_io_base, (pcr->pci_io_base & 0x00f0) << 8,
-		   pcr->pci_upper_io_limit, (pcr->pci_io_limit << 8) | 0x0fff);
-	} else {
+	    if ((pcr->pci_upper_io_base < pcr->pci_upper_io_limit) ||
+		((pcr->pci_upper_io_base == pcr->pci_upper_io_limit) &&
+		 ((pcr->pci_io_base & 0x00f0) <=
+		  (pcr->pci_io_limit & 0x00f0))))
+		printf("  IOBASE    0x%04x%04x  IOLIM 0x%04x%04x\n",
+		       pcr->pci_upper_io_base, (pcr->pci_io_base & 0x00f0) << 8,
+		       pcr->pci_upper_io_limit,
+		       (pcr->pci_io_limit << 8) | 0x0fff);
+	} else if ((pcr->pci_io_base & 0x00f0) <=
+		   (pcr->pci_io_limit & 0x00f0)) {
 	    printf("  IOBASE    0x%04x  IOLIM 0x%04x\n",
 		   (pcr->pci_io_base & 0x00f0) << 8,
 		   (pcr->pci_io_limit << 8) | 0x0fff);
 	}
     }
 
-    if (pcr->pci_mem_base || pcr->pci_mem_limit)
+    if ((pcr->pci_mem_base || pcr->pci_mem_limit) &&
+	((pcr->pci_mem_base & 0x00fff0) <= (pcr->pci_mem_limit & 0x00fff0)))
 	printf("  NOPREFETCH_MEMBASE 0x%08x  MEMLIM 0x%08x\n",
 	       (pcr->pci_mem_base & 0x00fff0) << 16,
 	       (pcr->pci_mem_limit << 16) | 0x0fffff);
@@ -479,147 +584,187 @@ print_bridge_pci_class(pciConfigPtr pcr)
 	pcr->pci_prefetch_upper_mem_limit) {
 	if (((pcr->pci_prefetch_mem_base & 0x0f) == 0x01) ||
 	    ((pcr->pci_prefetch_mem_limit & 0x0f) == 0x01)) {
-	    printf("  PREFETCH_MEMBASE   0x%08x%08x  MEMLIM 0x%08x%08x\n",
-		   (int)pcr->pci_prefetch_upper_mem_base,
-		   (pcr->pci_prefetch_mem_base & 0x00fff0) << 16,
-		   (int)pcr->pci_prefetch_upper_mem_limit,
-		   (pcr->pci_prefetch_mem_limit << 16) | 0x0fffff);
-	} else {
+	    if ((pcr->pci_prefetch_upper_mem_base <
+		 pcr->pci_prefetch_upper_mem_limit) ||
+		((pcr->pci_prefetch_upper_mem_base ==
+		  pcr->pci_prefetch_upper_mem_limit) &&
+		 ((pcr->pci_prefetch_mem_base & 0x00fff0) <=
+		  (pcr->pci_prefetch_mem_limit & 0x00fff0)))) {
+		printf("  PREFETCH_MEMBASE   0x%08x%08x  MEMLIM 0x%08x%08x\n",
+		       (int)pcr->pci_prefetch_upper_mem_base,
+		       (pcr->pci_prefetch_mem_base & 0x00fff0) << 16,
+		       (int)pcr->pci_prefetch_upper_mem_limit,
+		       (pcr->pci_prefetch_mem_limit << 16) | 0x0fffff);
+	    }
+	} else if ((pcr->pci_prefetch_mem_base & 0x00fff0) <=
+		   (pcr->pci_prefetch_mem_limit & 0x00fff0)) {
 	    printf("  PREFETCH_MEMBASE   0x%08x  MEMLIM 0x%08x\n",
 		   (pcr->pci_prefetch_mem_base & 0x00fff0) << 16,
 		   (pcr->pci_prefetch_mem_limit << 16) | 0x0fffff);
 	}
     }
 
-    printf("  %sFAST_B2B %sSEC_BUS_RST %sM_ABRT %sVGA_EN %sISA_EN"
-	   " %sSERR_EN %sPERR_EN\n",
-	   (pcr->pci_bridge_control & PCI_B_FAST_B_B) ? "" : "NO_",
-	   (pcr->pci_bridge_control & PCI_B_SB_RESET) ? "" : "NO_",
-	   (pcr->pci_bridge_control & PCI_B_M_ABORT) ? "" : "NO_",
-	   (pcr->pci_bridge_control & PCI_B_VGA_EN) ? "" : "NO_",
-	   (pcr->pci_bridge_control & PCI_B_ISA_EN) ? "" : "NO_",
-	   (pcr->pci_bridge_control & PCI_B_SERR_EN) ? "" : "NO_",
-	   (pcr->pci_bridge_control & PCI_B_P_ERR) ? "" : "NO_");
-}
-
-static void
-print_mach64(pciConfigPtr pcr)
-{
-    CARD32 sparse_io = 0;
-
-    printf(" CardVendor 0x%04x card 0x%04x\n",
-	   pcr->pci_subsys_vendor, pcr->pci_subsys_card);
-    printf("  STATUS    0x%04x  COMMAND 0x%04x\n",
-	   pcr->pci_status, pcr->pci_command);
-    printf("  CLASS     0x%02x 0x%02x 0x%02x  REVISION 0x%02x\n",
-	   pcr->pci_base_class, pcr->pci_sub_class,
-	   pcr->pci_prog_if, pcr->pci_rev_id);
-    printf("  BIST      0x%02x  HEADER 0x%02x  LATENCY 0x%02x  CACHE 0x%02x\n",
-	   pcr->pci_bist, pcr->pci_header_type, pcr->pci_latency_timer,
-	   pcr->pci_cache_line_size);
-    if (pcr->pci_base0)
-	printf("  APBASE    0x%08x  addr 0x%08x\n",
-	       (int)pcr->pci_base0, (int)(pcr->pci_base0 &
-		(pcr->pci_base0 & 0x1 ? 0xFFFFFFFC : 0xFFFFFFF0)));
-    if (pcr->pci_base1)
-	printf("  BLOCKIO   0x%08x  addr 0x%08x\n",
-	       (int)pcr->pci_base1, (int)(pcr->pci_base1 &
-		(pcr->pci_base1 & 0x1 ? 0xFFFFFFFC : 0xFFFFFFF0)));
-    if (pcr->pci_base2)
-	printf("  REGBASE   0x%08x  addr 0x%08x\n",
-	       (int)pcr->pci_base2, (int)(pcr->pci_base2 &
-		(pcr->pci_base2 & 0x1 ? 0xFFFFFFFC : 0xFFFFFFF0)));
-    if (pcr->pci_baserom)
-	printf("  BASEROM   0x%08x  addr 0x%08x  %sdecode-enabled\n",
-	       (int)pcr->pci_baserom, (int)(pcr->pci_baserom & 0xFFFF8000),
-	       pcr->pci_baserom & 0x1 ? "" : "not-");
-    if (pcr->pci_max_min_ipin_iline)
-	printf("  MAX_LAT   0x%02x  MIN_GNT 0x%02x"
-	       "  INT_PIN 0x%02x  INT_LINE 0x%02x\n",
-	       pcr->pci_max_lat, pcr->pci_min_gnt,
-	       pcr->pci_int_pin, pcr->pci_int_line);
-    switch (pcr->pci_user_config_0 & 0x03) {
-    case 0:
-	sparse_io = 0x2ec;
-	break;
-    case 1:
-	sparse_io = 0x1cc;
-	break;
-    case 2:
-	sparse_io = 0x1c8;
-	break;
+    if (pcr->basesize[0]) {
+	if ((pcr->pci_base0 & 0x7) == 0x4) {
+	    printf("  BASE0     0x%08x%08x  addr 0x%08x%08x",
+		   (int)pcr->pci_base1, (int)pcr->pci_base0,
+		   (int)pcr->pci_base1,
+		   (int)(pcr->pci_base0 &
+			 (pcr->pci_base0 & 0x1 ? 0xFFFFFFFC : 0xFFFFFFF0)));
+	    printf("  %ssize ",
+		   (pcr->minBasesize & (2 << 0)) ? "" : "estimated ");
+	    if (pcr->basesize[0] < 32)
+		printf("0x%08x", 1 << pcr->basesize[0]);
+	    else
+		printf("0x%08x00000000", 1 << (pcr->basesize[0] - 32));
+	    printf("  MEM%s 64BIT\n",
+		   (pcr->pci_base0 & 0x8) ? " PREFETCHABLE" : "");
+	} else {
+	    printf("  BASE0     0x%08x  addr 0x%08x",
+		   (int)pcr->pci_base0,
+		   (int)(pcr->pci_base0 &
+			 (pcr->pci_base0 & 0x1 ? 0xFFFFFFFC : 0xFFFFFFF0)));
+	    printf("  %ssize 0x%08x",
+		   (pcr->minBasesize & (2 << 0)) ? "" : "estimated ",
+		   1 << pcr->basesize[0]);
+	    printf("  %s%s\n",
+		   (pcr->pci_base0 & 0x1) ? "I/O" : "MEM",
+		   ((pcr->pci_base0 & 0x9) == 0x8) ? " PREFETCHABLE" : "");
+	}
     }
-    printf("  SPARSEIO  0x%03x    %s IO enabled    %sable 0x46E8\n",
-	   (int)sparse_io, pcr->pci_user_config_0 & 0x04 ? "Block" : "Sparse",
-	   pcr->pci_user_config_0 & 0x08 ? "Dis" : "En");
-}
 
-static void
-print_i128(pciConfigPtr pcr)
-{
-    printf(" CardVendor 0x%04x card 0x%04x\n",
-	   pcr->pci_subsys_vendor, pcr->pci_subsys_card);
-    printf("  STATUS    0x%04x  COMMAND 0x%04x\n",
-	   pcr->pci_status, pcr->pci_command);
-    printf("  CLASS     0x%02x 0x%02x 0x%02x  REVISION 0x%02x\n",
-	   pcr->pci_base_class, pcr->pci_sub_class, pcr->pci_prog_if,
-	   pcr->pci_rev_id);
-    printf("  BIST      0x%02x  HEADER 0x%02x  LATENCY 0x%02x  CACHE 0x%02x\n",
-	   pcr->pci_bist, pcr->pci_header_type, pcr->pci_latency_timer,
-	   pcr->pci_cache_line_size);
-    printf("  MW0_AD    0x%08x  addr 0x%08x  %spre-fetchable\n",
-	   (int)pcr->pci_base0, (int)(pcr->pci_base0 & 0xFFC00000),
-	   pcr->pci_base0 & 0x8 ? "" : "not-");
-    printf("  MW1_AD    0x%08x  addr 0x%08x  %spre-fetchable\n",
-	   (int)pcr->pci_base1, (int)(pcr->pci_base1 & 0xFFC00000),
-	   pcr->pci_base1 & 0x8 ? "" : "not-");
-    printf("  XYW_AD(A) 0x%08x  addr 0x%08x\n",
-	   (int)pcr->pci_base2, (int)(pcr->pci_base2 & 0xFFC00000));
-    printf("  XYW_AD(B) 0x%08x  addr 0x%08x\n",
-	   (int)pcr->pci_base3, (int)(pcr->pci_base3 & 0xFFC00000));
-    printf("  RBASE_G   0x%08x  addr 0x%08x\n",
-	   (int)pcr->pci_base4, (int)(pcr->pci_base4 & 0xFFFF0000));
-    printf("  IO        0x%08x  addr 0x%08x\n",
-	   (int)pcr->pci_base5, (int)(pcr->pci_base5 & 0xFFFFFF00));
-    printf("  RBASE_E   0x%08x  addr 0x%08x  %sdecode-enabled\n",
-	   (int)pcr->pci_baserom, (int)(pcr->pci_baserom & 0xFFFF8000),
-	   pcr->pci_baserom & 0x1 ? "" : "not-");
-    if (pcr->pci_max_min_ipin_iline)
-	printf("  MAX_LAT   0x%02x  MIN_GNT 0x%02x"
-	       "  INT_PIN 0x%02x  INT_LINE 0x%02x\n",
-	       pcr->pci_max_lat, pcr->pci_min_gnt,
+    if (pcr->basesize[1]) {
+	if ((pcr->pci_base1 & 0x7) == 0x4) {
+	    printf("  BASE1     0x%08x (invalid)\n", (int)pcr->pci_base0);
+	} else {
+	    printf("  BASE1     0x%08x  addr 0x%08x",
+		   (int)pcr->pci_base1,
+		   (int)(pcr->pci_base1 &
+			 (pcr->pci_base1 & 0x1 ? 0xFFFFFFFC : 0xFFFFFFF0)));
+	    printf("  %ssize 0x%08x",
+		   (pcr->minBasesize & (2 << 1)) ? "" : "estimated ",
+		   1 << pcr->basesize[1]);
+	    printf("  %s%s\n",
+		   (pcr->pci_base1 & 0x1) ? "I/O" : "MEM",
+		   ((pcr->pci_base1 & 0x9) == 0x8) ? " PREFETCHABLE" : "");
+	}
+    }
+
+    if (pcr->basesize[6]) {
+	printf("  BASEROM   0x%08x  addr 0x%08x",
+	       (int)pcr->pci_br_rom, (int)(pcr->pci_br_rom & 0xFFFFF800));
+	printf("  %ssize 0x%08x",
+	       (pcr->minBasesize & (2 << 7)) ? "" : "estimated ",
+	       1 << pcr->basesize[6]);
+	printf("  %senabled\n",
+	       pcr->pci_br_rom & 0x1 ? "" : "not ");
+    }
+
+    printf("  %sFAST_B2B %sSEC_BUS_RST %sM_ABRT %sVGA16 %sVGA_EN %sISA_EN"
+	   " %sSERR_EN %sPERR_EN\n",
+	   (pcr->pci_bridge_control & PCI_PCI_BRIDGE_FAST_B2B_EN) ?
+		"" : "NO_",
+	   (pcr->pci_bridge_control & PCI_PCI_BRIDGE_SECONDARY_RESET) ?
+		"" : "NO_",
+	   (pcr->pci_bridge_control & PCI_PCI_BRIDGE_MASTER_ABORT_EN) ?
+		"" : "NO_",
+	   (pcr->pci_bridge_control & PCI_PCI_BRIDGE_VGA16_EN) ?
+		"" : "NO_",
+	   (pcr->pci_bridge_control & PCI_PCI_BRIDGE_VGA_EN) ?
+		"" : "NO_",
+	   (pcr->pci_bridge_control & PCI_PCI_BRIDGE_ISA_EN) ?
+		"" : "NO_",
+	   (pcr->pci_bridge_control & PCI_PCI_BRIDGE_SERR_EN) ?
+		"" : "NO_",
+	   (pcr->pci_bridge_control & PCI_PCI_BRIDGE_PARITY_EN) ?
+		"" : "NO_");
+
+    if (pcr->pci_int_pin || pcr->pci_int_line)
+	printf("  INT_PIN 0x%02x  INT_LINE 0x%02x\n",
 	       pcr->pci_int_pin, pcr->pci_int_line);
 }
 
 static void
-print_dc21050(pciConfigPtr pcr)
+print_header_type_2(pciConfigPtr pcr)
 {
-    printf("  STATUS    0x%04x  COMMAND 0x%04x\n",
-	   pcr->pci_status, pcr->pci_command);
-    printf("  CLASS     0x%02x 0x%02x 0x%02x  REVISION 0x%02x\n",
-	   pcr->pci_base_class, pcr->pci_sub_class, pcr->pci_prog_if,
-	   pcr->pci_rev_id);
-    printf("  BIST      0x%02x  HEADER 0x%02x  LATENCY 0x%02x  CACHE 0x%02x\n",
-	   pcr->pci_bist, pcr->pci_header_type, pcr->pci_latency_timer,
-	   pcr->pci_cache_line_size);
-    printf("  PRIBUS    0x%02x  SECBUS 0x%02x  SUBBUS 0x%02x  SECLT 0x%02x\n",
-	   pcr->pci_primary_bus_number, pcr->pci_secondary_bus_number,
-	   pcr->pci_subordinate_bus_number, pcr->pci_secondary_latency_timer);
-    printf("  IOBASE    0x%02x  IOLIM 0x%02x  SECSTATUS 0x%04x\n",
-	   pcr->pci_io_base << 8, (pcr->pci_io_limit << 8) | 0xfff,
-	   pcr->pci_secondary_status);
-    printf("  NOPREFETCH_MEMBASE 0x%08x  MEMLIM 0x%08x\n",
-	   pcr->pci_mem_base << 16, (pcr->pci_mem_limit << 16) | 0xfffff);
-    printf("  PREFETCH_MEMBASE   0x%08x  MEMLIM 0x%08x\n",
-	   pcr->pci_prefetch_mem_base << 16,
-	   (pcr->pci_prefetch_mem_limit << 16) | 0xfffff);
-    printf("  RBASE_E   0x%08x  addr 0x%08x  %sdecode-enabled\n",
-	   (int)pcr->pci_baserom, (int)(pcr->pci_baserom & 0xFFFF8000),
-	   pcr->pci_baserom & 0x1 ? "" : "not-");
-    if (pcr->pci_max_min_ipin_iline)
-	printf("  MAX_LAT   0x%02x  MIN_GNT 0x%02x"
-	       "  INT_PIN 0x%02x  INT_LINE 0x%02x\n",
-	       pcr->pci_max_lat, pcr->pci_min_gnt,
+    printf("  PRIBUS    0x%02x  SECBUS 0x%02x  SUBBUS 0x%02x\n",
+	   pcr->pci_cb_primary_bus_number, pcr->pci_cb_cardbus_bus_number,
+	   pcr->pci_cb_subordinate_bus_number);
+    printf("  SECLT     0x%02x  SECSTATUS 0x%04x\n",
+	   pcr->pci_cb_latency_timer, pcr->pci_cb_secondary_status);
+
+    if ((pcr->pci_cb_membase0 || pcr->pci_cb_memlimit0) &&
+	((pcr->pci_cb_membase0 & 0xFFFFF000) <=
+	 (pcr->pci_cb_memlimit0 & 0xFFFFF000)))
+	printf("  MEMBASE0 0x%08x  MEMLIM0 0x%08x%s\n",
+	       (int)pcr->pci_cb_membase0 & 0xFFFFF000,
+	       (int)pcr->pci_cb_memlimit0 | 0x00000FFF,
+	       (pcr->pci_bridge_control & PCI_CB_BRIDGE_CTL_PREFETCH_MEM0) ?
+		" PREFETCHABLE" : "");
+
+    if ((pcr->pci_cb_membase1 || pcr->pci_cb_memlimit1) &&
+	((pcr->pci_cb_membase1 & 0xFFFFF000) <=
+	 (pcr->pci_cb_memlimit1 & 0xFFFFF000)))
+	printf("  MEMBASE1 0x%08x  MEMLIM1 0x%08x%s\n",
+	       (int)pcr->pci_cb_membase1 & 0xFFFFF000,
+	       (int)pcr->pci_cb_memlimit1 | 0x00000FFF,
+	       (pcr->pci_bridge_control & PCI_CB_BRIDGE_CTL_PREFETCH_MEM1) ?
+		" PREFETCHABLE" : "");
+
+    if ((pcr->pci_cb_iobase0 || pcr->pci_cb_iolimit0) &&
+	((pcr->pci_cb_iobase0 & 0xFFFFFFFC) <=
+	 (pcr->pci_cb_iolimit0 & 0xFFFFFFFC)))
+	printf("  IOBASE0  0x%08x      IOLIM0  0x%08x\n",
+	       (int)pcr->pci_cb_iobase0 & 0xFFFFFFFC,
+	       (int)pcr->pci_cb_iolimit0 | 0x00000003);
+
+    if ((pcr->pci_cb_iobase1 || pcr->pci_cb_iolimit1) &&
+	((pcr->pci_cb_iobase1 & 0xFFFFFFFC) <=
+	 (pcr->pci_cb_iolimit1 & 0xFFFFFFFC)))
+	printf("  IOBASE1  0x%08x      IOLIM1  0x%08x\n",
+	       (int)pcr->pci_cb_iobase1 & 0xFFFFFFFC,
+	       (int)pcr->pci_cb_iolimit1 | 0x00000003);
+
+    if (pcr->basesize[0]) {
+	if ((pcr->pci_base0 & 0x7) == 0x4) {
+	    printf("  BASE0     0x%08x (invalid)\n", (int)pcr->pci_base0);
+	} else {
+	    printf("  BASE0     0x%08x  addr 0x%08x",
+		   (int)pcr->pci_base0,
+		   (int)(pcr->pci_base0 &
+			 (pcr->pci_base0 & 0x1 ? 0xFFFFFFFC : 0xFFFFFFF0)));
+	    printf("  %ssize 0x%08x",
+		   (pcr->minBasesize & (2 << 0)) ? "" : "estimated ",
+		   1 << pcr->basesize[0]);
+	    printf("  %s%s\n",
+		   (pcr->pci_base0 & 0x1) ? "I/O" : "MEM",
+		   ((pcr->pci_base0 & 0x9) == 0x8) ? " PREFETCHABLE" : "");
+	}
+    }
+
+    printf("  %s16BIT_INT %sSEC_BUS_RST %sM_ABRT %sVGA16 %sVGA_EN %sISA_EN"
+	   " %sSERR_EN %sPERR_EN %sPOST_WRITES\n",
+	   (pcr->pci_bridge_control & PCI_CB_BRIDGE_CTL_16BIT_INT) ?
+		"" : "NO_",
+	   (pcr->pci_bridge_control & PCI_PCI_BRIDGE_SECONDARY_RESET) ?
+		"" : "NO_",
+	   (pcr->pci_bridge_control & PCI_PCI_BRIDGE_MASTER_ABORT_EN) ?
+		"" : "NO_",
+	   (pcr->pci_bridge_control & PCI_PCI_BRIDGE_VGA16_EN) ?
+		"" : "NO_",
+	   (pcr->pci_bridge_control & PCI_PCI_BRIDGE_VGA_EN) ?
+		"" : "NO_",
+	   (pcr->pci_bridge_control & PCI_PCI_BRIDGE_ISA_EN) ?
+		"" : "NO_",
+	   (pcr->pci_bridge_control & PCI_PCI_BRIDGE_SERR_EN) ?
+		"" : "NO_",
+	   (pcr->pci_bridge_control & PCI_PCI_BRIDGE_PARITY_EN) ?
+		"" : "NO_",
+	   (pcr->pci_bridge_control & PCI_CB_BRIDGE_CTL_POST_WRITES) ?
+		"" : "NO_");
+
+    if (pcr->pci_int_pin || pcr->pci_int_line)
+	printf("  INT_PIN 0x%02x  INT_LINE 0x%02x\n",
 	       pcr->pci_int_pin, pcr->pci_int_line);
 }
 
@@ -629,28 +774,8 @@ print_simba(pciConfigPtr pcr)
     int   i;
     CARD8 io, mem;
 
-    printf("  STATUS    0x%04x  COMMAND 0x%04x\n",
-	   pcr->pci_status, pcr->pci_command);
-    printf("  CLASS     0x%02x 0x%02x 0x%02x  REVISION 0x%02x\n",
-	   pcr->pci_base_class, pcr->pci_sub_class, pcr->pci_prog_if,
-	   pcr->pci_rev_id);
-    printf("  HEADER    0x%02x  LATENCY 0x%02x  CACHE 0x%02x\n",
-	   pcr->pci_header_type, pcr->pci_latency_timer,
-	   pcr->pci_cache_line_size);
-    printf("  PRIBUS    0x%02x  SECBUS 0x%02x  SUBBUS 0x%02x  SECLT 0x%02x\n",
-	   pcr->pci_primary_bus_number, pcr->pci_secondary_bus_number,
-	   pcr->pci_subordinate_bus_number, pcr->pci_secondary_latency_timer);
-    printf("  SECSTATUS 0x%04x\n",
-	   pcr->pci_secondary_status);
-    printf("  %sFAST_B2B %sSEC_BUS_RST %sM_ABRT %sVGA_EN %sISA_EN"
-	   " %sSERR_EN %sPERR_EN\n",
-	   (pcr->pci_bridge_control & PCI_B_FAST_B_B) ? "" : "NO_",
-	   (pcr->pci_bridge_control & PCI_B_SB_RESET) ? "" : "NO_",
-	   (pcr->pci_bridge_control & PCI_B_M_ABORT) ? "" : "NO_",
-	   (pcr->pci_bridge_control & PCI_B_VGA_EN) ? "" : "NO_",
-	   (pcr->pci_bridge_control & PCI_B_ISA_EN) ? "" : "NO_",
-	   (pcr->pci_bridge_control & PCI_B_SERR_EN) ? "" : "NO_",
-	   (pcr->pci_bridge_control & PCI_B_P_ERR) ? "" : "NO_");
+    print_header_type_1(pcr);
+
     printf("  TICK      0x%08lx         SECCNTL 0x%02x\n", (long)
 	   pciReadLong(pcr->tag, 0x00b0), pciReadByte(pcr->tag, 0x00dd));
     printf("  MASTER RETRIES:  PRIMARY 0x%02x,  SECONDARY 0x%02x\n",
@@ -679,10 +804,10 @@ print_simba(pciConfigPtr pcr)
 	   (mem = pciReadByte(pcr->tag, 0x00df)));
     for (i = 0;  i < 8;  i++)
 	if (io & (1 << i))
-	    printf("  BUS I/O  0x%06x-0x%06x\n", i << 21, ((i + 1) << 21) - 1);
+	    printf("  BUS I/O   0x%06x-0x%06x\n", i << 21, ((i + 1) << 21) - 1);
     for (i = 0;  i < 8;  i++)
 	if (mem & (1 << i))
-	    printf("  BUS MEM  0x%08x-0x%08x\n", i << 29, ((i + 1) << 29) - 1);
+	    printf("  BUS MEM   0x%08x-0x%08x\n", i << 29, ((i + 1) << 29) - 1);
 }
 
 static int cbn_460gx = -1;
@@ -692,16 +817,9 @@ print_460gx_sac(pciConfigPtr pcr)
 {
     CARD32 tmp;
 
-    /* Print generalities */
-    printf("  STATUS    0x%04x  COMMAND 0x%04x\n",
-	   pcr->pci_status, pcr->pci_command);
-    printf("  CLASS     0x%02x 0x%02x 0x%02x  REVISION 0x%02x\n",
-	   pcr->pci_base_class, pcr->pci_sub_class, pcr->pci_prog_if,
-	   pcr->pci_rev_id);
-
     tmp = pcr->pci_user_config;
     pcr->pci_user_config = 0;
-    print_default_class(pcr);
+    print_header_type_0(pcr);
     pcr->pci_user_config = tmp;
 
     /* Only print what XFree86 might be interested in */
@@ -752,16 +870,9 @@ print_460gx_pxb(pciConfigPtr pcr)
 {
     CARD32 tmp;
 
-    /* Print generalities */
-    printf("  STATUS    0x%04x  COMMAND 0x%04x\n",
-	   pcr->pci_status, pcr->pci_command);
-    printf("  CLASS     0x%02x 0x%02x 0x%02x  REVISION 0x%02x\n",
-	   pcr->pci_base_class, pcr->pci_sub_class, pcr->pci_prog_if,
-	   pcr->pci_rev_id);
-
     tmp = pcr->pci_user_config;
     pcr->pci_user_config = 0;
-    print_default_class(pcr);
+    print_header_type_0(pcr);
     pcr->pci_user_config = tmp;
 
     /* Only print what XFree86 might be interested in */
@@ -774,16 +885,9 @@ print_460gx_gxb(pciConfigPtr pcr)
 {
     CARD32 tmp;
 
-    /* Print generalities */
-    printf("  STATUS    0x%04x  COMMAND 0x%04x\n",
-	   pcr->pci_status, pcr->pci_command);
-    printf("  CLASS     0x%02x 0x%02x 0x%02x  REVISION 0x%02x\n",
-	   pcr->pci_base_class, pcr->pci_sub_class, pcr->pci_prog_if,
-	   pcr->pci_rev_id);
-
     tmp = pcr->pci_user_config;
     pcr->pci_user_config = 0;
-    print_default_class(pcr);
+    print_header_type_0(pcr);
     pcr->pci_user_config = tmp;
 
     /* Only print what XFree86 might be interested in */
