@@ -1,4 +1,4 @@
-/* $XFree86: xc/programs/Xserver/hw/xfree86/os-support/bus/Pci.h,v 1.53 2006/04/18 15:57:25 tsi Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/os-support/bus/Pci.h,v 1.59 2008/03/27 18:01:44 tsi Exp $ */
 /*
  * Copyright 1998 by Concurrent Computer Corporation
  *
@@ -134,7 +134,8 @@
  */
 #define MAX_PCI_DEVICES 128	/* Max number of devices accomodated */
 				/* by xf86scanpci		     */
-#if defined(sun) && defined(SVR4) && defined(sparc)
+#if (defined(sparc) || defined(__sparc__)) && \
+    ((defined(sun) && defined(SVR4)) || defined(__OpenBSD__))
 # define MAX_PCI_BUSES   4096	/* Max number of PCI buses           */
 #elif defined(__alpha__) && defined (linux)
 # define MAX_PCI_DOMAINS	512
@@ -167,6 +168,7 @@
 #define PCI_FUNC_FROM_TAG(tag) (((tag) & 0x00000700u) >> 8)
 
 #define PCI_DFN_FROM_TAG(tag)  (((tag) & 0x0000ff00u) >> 8)
+#define PCI_BDF_FROM_TAG(tag)  (((tag) & 0x00ffff00u) >> 8)
 #define PCI_BDEV_FROM_TAG(tag) ((tag) & 0x00fff800u)
 
 #define PCI_DOM_FROM_BUS(bus)  (((bus) >> 8) & (PCI_DOM_MASK))
@@ -242,10 +244,17 @@
 
 #define PCI_CFGMECH1_MAXDEV	32
 
+/*
+ * CAN_HARDFAIL_MASTER_ABORTS can be #define'd to a Bool function or some
+ * other boolean expression should the need arise.
+ */
+#undef CAN_HARDFAIL_MASTER_ABORTS
+
 /*
  * Select architecture specific PCI init function
  */
 #if defined(__alpha__)
+# define CAN_HARDFAIL_MASTER_ABORTS FALSE	/* Uncertain */
 # if defined(linux)
 #  define ARCH_PCI_INIT axpPciInit
 #  define INCLUDE_XF86_MAP_PCI_MEM
@@ -259,18 +268,21 @@
 #  define INCLUDE_XF86_NO_DOMAIN
 # endif
 #elif defined(__arm__)
+# define CAN_HARDFAIL_MASTER_ABORTS TRUE	/* Unknown */
 # if defined(linux)
 #  define ARCH_PCI_INIT linuxPciInit
 #  define INCLUDE_XF86_MAP_PCI_MEM
 #  define INCLUDE_XF86_NO_DOMAIN
 # endif
 #elif defined(__hppa__)
+# define CAN_HARDFAIL_MASTER_ABORTS TRUE	/* Unknown */
 # if defined(linux)
 #  define ARCH_PCI_INIT linuxPciInit
 #  define INCLUDE_XF86_MAP_PCI_MEM
 #  define INCLUDE_XF86_NO_DOMAIN
 # endif
 #elif defined(__ia64__)
+# define CAN_HARDFAIL_MASTER_ABORTS TRUE	/* Broken */
 # if defined(linux)
 #  define ARCH_PCI_INIT linuxPciInit
 #  define INCLUDE_XF86_MAP_PCI_MEM
@@ -282,6 +294,7 @@
 # endif
 # define XF86SCANPCI_WRAPPER ia64ScanPCIWrapper
 #elif defined(__i386__) || defined(i386)
+# define CAN_HARDFAIL_MASTER_ABORTS FALSE	/* Sane (so far) */
 # define ARCH_PCI_INIT ix86PciInit
 # define INCLUDE_XF86_MAP_PCI_MEM
 # define INCLUDE_XF86_NO_DOMAIN
@@ -289,18 +302,21 @@
 #  define ARCH_PCI_OS_INIT linuxPciInit
 # endif
 #elif defined(__mc68000__)
+# define CAN_HARDFAIL_MASTER_ABORTS TRUE	/* Unknown */
 # if defined(linux)
 #  define ARCH_PCI_INIT linuxPciInit
 #  define INCLUDE_XF86_MAP_PCI_MEM
 #  define INCLUDE_XF86_NO_DOMAIN
 # endif
 #elif defined(__mips__)
+# define CAN_HARDFAIL_MASTER_ABORTS TRUE	/* Unknown */
 # if defined(linux)
 #  define ARCH_PCI_INIT linuxPciInit
 #  define INCLUDE_XF86_MAP_PCI_MEM
 #  define INCLUDE_XF86_NO_DOMAIN
 # endif
 #elif defined(__powerpc__)
+# define CAN_HARDFAIL_MASTER_ABORTS TRUE	/* Broken */
 # if defined(linux)
 #  define ARCH_PCI_INIT linuxPciInit
 #  define INCLUDE_XF86_MAP_PCI_MEM
@@ -321,33 +337,43 @@
 #  define INCLUDE_XF86_NO_DOMAIN
 # endif
 #elif defined(__s390__)
+# define CAN_HARDFAIL_MASTER_ABORTS TRUE	/* Unknown */
 # if defined(linux)
 #  define ARCH_PCI_INIT linuxPciInit
 #  define INCLUDE_XF86_MAP_PCI_MEM
 #  define INCLUDE_XF86_NO_DOMAIN
 # endif
 #elif defined(__sh__)
+# define CAN_HARDFAIL_MASTER_ABORTS TRUE	/* Unknown */
 # if defined(linux)
 #  define ARCH_PCI_INIT linuxPciInit
 #  define INCLUDE_XF86_MAP_PCI_MEM
 #  define INCLUDE_XF86_NO_DOMAIN
 # endif
 #elif defined(__sparc__) || defined(sparc)
+# define CAN_HARDFAIL_MASTER_ABORTS TRUE	/* Broken */
 # if defined(linux)
 #  define ARCH_PCI_INIT linuxPciInit
 #  define INCLUDE_XF86_MAP_PCI_MEM
 # elif defined(sun)
 #  define ARCH_PCI_INIT sparcPciInit
 #  define INCLUDE_XF86_MAP_PCI_MEM
-# elif (defined(__OpenBSD__) || defined(__FreeBSD__)) && defined(__sparc64__)
-#  define  ARCH_PCI_INIT freebsdPciInit
+# elif defined(__FreeBSD__)
+#  define ARCH_PCI_INIT freebsdPciInit
 #  define INCLUDE_XF86_MAP_PCI_MEM
 #  define INCLUDE_XF86_NO_DOMAIN
+# elif defined(__NetBSD__)
+#  define ARCH_PCI_INIT netbsdPciInit
+#  define INCLUDE_XF86_MAP_PCI_MEM
+#  define INCLUDE_XF86_NO_DOMAIN
+# elif defined(__OpenBSD__)
+#  define ARCH_PCI_INIT sparcPciInit
+#  define INCLUDE_XF86_MAP_PCI_MEM
+#  define ARCH_PCI_OS_INIT freebsdPciInit
 # endif
-# if !defined(__FreeBSD__) && !defined(__OpenBSD__)
-#  define ARCH_PCI_PCI_BRIDGE sparcPciPciBridge
-# endif
+# define ARCH_PCI_PCI_BRIDGE sparcPciPciBridge
 #elif defined(__amd64__) || defined(__x86_64__)
+# define CAN_HARDFAIL_MASTER_ABORTS FALSE	/* Likely */
 # if defined(__FreeBSD__)
 #  define ARCH_PCI_INIT freebsdPciInit
 # else
@@ -364,6 +390,10 @@
 #error No PCI support available for this architecture/OS combination
 #endif
 
+#ifndef CAN_HARDFAIL_MASTER_ABORTS
+#define CAN_HARDFAIL_MASTER_ABORTS TRUE		/* Assume the worst */
+#endif
+
 extern void ARCH_PCI_INIT(void);
 #if defined(ARCH_PCI_OS_INIT)
 extern void ARCH_PCI_OS_INIT(void);
@@ -416,6 +446,7 @@ typedef struct pci_bus_info {
 	pciBusFuncs_p  funcs;        /* PCI access functions        */
 	void          *pciBusPriv;   /* Implementation private data */
 	pciConfigPtr   bridge;       /* bridge that opens this bus  */
+	int            pciMaxOffset; /* Size of config for one device */
 } pciBusInfo_t;
 
 #define HOST_NO_BUS ((pciBusInfo_t *)(-1))
