@@ -1,4 +1,4 @@
-/* $XFree86: xc/programs/Xserver/hw/xfree86/os-support/bus/ix86Pci.c,v 1.26 2004/02/13 23:58:47 dawes Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/os-support/bus/ix86Pci.c,v 1.28 2008/05/14 15:40:49 tsi Exp $ */
 /*
  * ix86Pci.c - x86 PCI driver
  *
@@ -300,7 +300,8 @@ void ix86PciSelectCfgmech(void)
 #ifdef DEBUGPCI
       if (xf86Verbose > 2) {
 	ErrorF("Checking config type 1:\n"
-		"\tinitial value of MODE1_ADDR_REG is 0x%08x\n", oldVal1);
+		"\tinitial value of MODE1_ADDR_REG is 0x%08lx\n",
+		(unsigned long)oldVal1);
 	ErrorF("\tChecking that all bits in mask 0x7f000000 are clear\n");
       }
 #endif
@@ -337,8 +338,8 @@ void ix86PciSelectCfgmech(void)
 
 #ifdef DEBUGPCI
 	if (xf86Verbose > 2) {
-	    ErrorF("\tValue read back from MODE1_ADDR_REG is 0x%08x\n",
-			mode1Res1);
+	    ErrorF("\tValue read back from MODE1_ADDR_REG is 0x%08lx\n",
+		   (unsigned long)mode1Res1);
 	    ErrorF("\tRestoring original contents of MODE1_ADDR_REG\n");
 	}
 #endif
@@ -392,8 +393,8 @@ void ix86PciSelectCfgmech(void)
 
 #ifdef DEBUGPCI
 	if (xf86Verbose > 2) {
-	    ErrorF("\tValue read back from MODE1_ADDR_REG is 0x%08x\n",
-			mode1Res2);
+	    ErrorF("\tValue read back from MODE1_ADDR_REG is 0x%08lx\n",
+		   (unsigned long)mode1Res2);
 	    ErrorF("\tRestoring original contents of MODE1_ADDR_REG\n");
 	}
 #endif
@@ -569,6 +570,9 @@ ix86PciReadLongCFG1(PCITAG Tag, int reg)
     ErrorF("ix86PciReadLong 0x%lx, %d\n", Tag, reg);
 #endif
 
+    if ((reg < 0) || (reg >= 256))
+	return PCI_NOT_FOUND;
+
     addr = PCI_ADDR_FROM_TAG_CFG1(Tag,reg);
     outl(PCI_CFGMECH1_ADDRESS_REG, addr);
     data = inl(PCI_CFGMECH1_DATA_REG);
@@ -591,6 +595,9 @@ ix86PciReadLongCFG2(PCITAG Tag, int reg)
     ErrorF("ix86PciReadLong 0x%lx, %d\n", Tag, reg);
 #endif
 
+    if ((reg < 0) || (reg >= 256))
+	return PCI_NOT_FOUND;
+
     forward  = PCI_FORWARD_FROM_TAG(Tag);
     enable   = PCI_ENABLE_FROM_TAG(Tag);
     addr     = PCI_ADDR_FROM_TAG_CFG2(Tag,reg);
@@ -620,6 +627,9 @@ ix86PciWriteLongCFG1(PCITAG Tag, int reg, CARD32 data)
 {
     CARD32    addr;
 
+    if ((reg < 0) || (reg >= 256))
+	return;
+
     addr = PCI_ADDR_FROM_TAG_CFG1(Tag,reg);
     outl(PCI_CFGMECH1_ADDRESS_REG, addr);
     outl(PCI_CFGMECH1_DATA_REG, data);
@@ -632,6 +642,9 @@ ix86PciWriteLongCFG2(PCITAG Tag, int reg, CARD32 data)
     CARD32    addr;
     CARD8 forward, enable;
 
+    if ((reg < 0) || (reg >= 256))
+	return;
+
     forward  = PCI_FORWARD_FROM_TAG(Tag);
     enable   = PCI_ENABLE_FROM_TAG(Tag);
     addr     = PCI_ADDR_FROM_TAG_CFG2(Tag,reg);
@@ -659,6 +672,9 @@ ix86PciSetBitsLongCFG1(PCITAG Tag, int reg, CARD32 mask, CARD32 val)
     ErrorF("ix86PciSetBitsLong 0x%lx, %d\n", Tag, reg);
 #endif
 
+    if ((reg < 0) || (reg >= 256))
+	return;
+
     addr = PCI_ADDR_FROM_TAG_CFG1(Tag,reg);
     outl(PCI_CFGMECH1_ADDRESS_REG, addr);
     data = inl(PCI_CFGMECH1_DATA_REG);
@@ -677,6 +693,9 @@ ix86PciSetBitsLongCFG2(PCITAG Tag, int reg, CARD32 mask, CARD32 val)
     ErrorF("ix86PciSetBitsLong 0x%lx, %d\n", Tag, reg);
 #endif
 
+    if ((reg < 0) || (reg >= 256))
+	return;
+
     forward  = PCI_FORWARD_FROM_TAG(Tag);
     enable   = PCI_ENABLE_FROM_TAG(Tag);
     addr     = PCI_ADDR_FROM_TAG_CFG2(Tag,reg);
