@@ -1,6 +1,6 @@
-/* $XFree86: xc/programs/Xserver/hw/xfree86/os-support/bus/sparcPci.c,v 1.22 2006/02/18 03:31:38 dawes Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/os-support/bus/sparcPci.c,v 1.33 2008/06/23 14:42:22 tsi Exp $ */
 /*
- * Copyright (C) 2001-2005 The XFree86 Project, Inc.
+ * Copyright (C) 2001-2007 The XFree86 Project, Inc.
  * All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining
@@ -53,10 +53,16 @@
 #include "Pci.h"
 #include "xf86sbusBus.h"
 
-#if defined(sun)
+#if defined(sun) || defined(__OpenBSD__)
 
+#if defined(__OpenBSD__)
+#define apertureDevName "/dev/mem"
+#else
 extern char *apertureDevName;
-static int  apertureFd = -1;
+#endif
+
+static unsigned long long pagemask;
+static int apertureFd = -1;
 
 /*
  * A version of xf86MapVidMem() that allows for 64-bit displacements (but not
@@ -77,9 +83,10 @@ sparcMapAperture(int iScreen, int Flags,
 	lastFlags = Flags;
 	apertureFd = open(apertureDevName,
 	    (Flags & VIDMEM_READONLY) ? O_RDONLY : O_RDWR);
-	if (apertureFd < 0)
-	    FatalError("sparcMapAperture:  open failure:  %s\n",
-		       strerror(errno));
+	if (apertureFd < 0) {
+	    ErrorF("sparcMapAperture:  open failure:  %s\n", strerror(errno));
+	    return MAP_FAILED;
+	}
     }
 
     result = mmap(NULL, Size,
@@ -87,8 +94,16 @@ sparcMapAperture(int iScreen, int Flags,
 		      PROT_READ : (PROT_READ | PROT_WRITE),
 		  MAP_SHARED, apertureFd, (off_t)Base);
 
-    if (result == MAP_FAILED)
-	FatalError("sparcMapAperture:  mmap failure:  %s\n", strerror(errno));
+    if (result == MAP_FAILED) {
+	ErrorF("sparcMapAperture:  mmap failure:  %s\n", strerror(errno));
+	return MAP_FAILED;
+    }
+
+#if !defined(sun) && !defined(__bsdi__) && \
+    !(defined(MACH) && defined(__GNU__))
+    /* Register this mapping */
+    xf86MakeNewMapping(iScreen, Flags, Base, Size, result);
+#endif
 
     return result;
 }
@@ -101,7 +116,8 @@ typedef struct _sparcDomainRec {
     unsigned long long mem_addr, mem_size;
     pointer pci, io;
     int bus_min, bus_max;
-    unsigned char dfn_mask[256 / 8];
+    int tagMultiplier, maxOffset;
+    unsigned char *bdf_mask;
 } sparcDomainRec, *sparcDomainPtr;
 
 #define SetBitInMap(bit, map) \
@@ -122,26 +138,88 @@ static sparcDomainPtr xf86DomainInfo[MAX_DOMAINS];
 static int            pciNumDomains = 1;
 
 /* Variables to which this is assigned must be declared volatile */
-#define PciReg(base, tag, off, type) \
-    *(volatile type *)(pointer)((char *)(base) + \
-	(PCI_TAG_NO_DOMAIN(tag) | (off)))
+#define PciReg(domain, tag, off, type) \
+    *(volatile type *)(pointer)((char *)((domain)->pci) + \
+	((PCI_TAG_NO_DOMAIN(tag) * (domain)->tagMultiplier) | (off)))
+
+/*
+ * Functions to wrap PciReg accesses with membar's for fault isolation
+ * purposes.
+ */
+
+static __inline__ CARD8
+GetPciByte(sparcDomainPtr pDomain, PCITAG tag, int off)
+{
+    volatile CARD8 result;			/* Must be volatile */
+
+    barrier();
+    result = PciReg(pDomain, tag, off, CARD8);
+    barrier();
+    return result;
+}
+
+static __inline__ CARD16
+GetPciWord(sparcDomainPtr pDomain, PCITAG tag, int off)
+{
+    volatile CARD16 result;			/* Must be volatile */
+
+    barrier();
+    result = PciReg(pDomain, tag, off, CARD16);
+    barrier();
+    return result;
+}
+
+static __inline__ CARD32
+GetPciLong(sparcDomainPtr pDomain, PCITAG tag, int off)
+{
+    volatile CARD32 result;			/* Must be volatile */
+
+    barrier();
+    result = PciReg(pDomain, tag, off, CARD32);
+    barrier();
+    return result;
+}
+
+static __inline__ void
+PutPciByte(sparcDomainPtr pDomain, PCITAG tag, int off, CARD8 val)
+{
+    barrier();
+    PciReg(pDomain, tag, off, CARD8) = val;
+    barrier();
+}
+
+static __inline__ void
+PutPciWord(sparcDomainPtr pDomain, PCITAG tag, int off, CARD16 val)
+{
+    barrier();
+    PciReg(pDomain, tag, off, CARD16) = val;
+    barrier();
+}
+
+static __inline void
+PutPciLong(sparcDomainPtr pDomain, PCITAG tag, int off, CARD32 val)
+{
+    barrier();
+    PciReg(pDomain, tag, off, CARD32) = val;
+    barrier();
+}
 
 /* Generic SPARC PCI access functions */
 static CARD32
 sparcPciCfgRead32(PCITAG tag, int off)
 {
-    pciBusInfo_t    *pBusInfo;
-    sparcDomainPtr  pDomain;
-    volatile CARD32 result = (CARD32)(-1);	/* Must be volatile */
-    int             bus;
+    pciBusInfo_t   *pBusInfo;
+    sparcDomainPtr pDomain;
+    CARD32         result = (CARD32)(-1);
+    int            bus;
 
-    if ((off >= 0) && (off <= 252) && !(off & 3) &&
+    if ((off >= 0) && !(off & 3) &&
 	((bus = PCI_BUS_FROM_TAG(tag)) < pciNumBuses) &&
 	(pBusInfo = pciBusInfo[bus]) && (pDomain = pBusInfo->pciBusPriv) &&
+	(off < pDomain->maxOffset) &&
 	(bus >= pDomain->bus_min) && (bus < pDomain->bus_max) &&
-	((bus > pDomain->bus_min) ||
-	 IsBitSetInMap(PCI_DFN_FROM_TAG(tag), pDomain->dfn_mask))) {
-	result = PciReg(pDomain->pci, tag, off, CARD32);
+	IsBitSetInMap(PCI_BDF_FROM_TAG(tag), pDomain->bdf_mask)) {
+	result = GetPciLong(pDomain, tag, off);
 
 	result = PCI_CPU(result);
     }
@@ -156,16 +234,16 @@ sparcPciCfgWrite32(PCITAG tag, int off, CARD32 val)
     sparcDomainPtr pDomain;
     int            bus;
 
-    if ((off < 0) || (off > 252) || (off & 3) ||
+    if ((off < 0) || (off & 3) ||
 	((bus = PCI_BUS_FROM_TAG(tag)) >= pciNumBuses) ||
 	!(pBusInfo = pciBusInfo[bus]) || !(pDomain = pBusInfo->pciBusPriv) ||
+	(off >= pDomain->maxOffset) ||
 	(bus < pDomain->bus_min) || (bus >= pDomain->bus_max) ||
-	((bus == pDomain->bus_min) &&
-	 !IsBitSetInMap(PCI_DFN_FROM_TAG(tag), pDomain->dfn_mask)))
+	!IsBitSetInMap(PCI_BDF_FROM_TAG(tag), pDomain->bdf_mask))
 	return;
 
     val = PCI_CPU(val);
-    PciReg(pDomain->pci, tag, off, CARD32) = val;
+    PutPciLong(pDomain, tag, off, val);
 }
 
 static void
@@ -195,10 +273,10 @@ static pciBusFuncs_t sparcPCIFunctions =
 static CARD32
 sabrePciCfgRead32(PCITAG tag, int off)
 {
-    pciBusInfo_t    *pBusInfo;
-    sparcDomainPtr  pDomain;
-    volatile CARD32 result;			/* Must be volatile */
-    int             bus;
+    pciBusInfo_t   *pBusInfo;
+    sparcDomainPtr pDomain;
+    CARD32         result;
+    int            bus;
 
     if (PCI_BDEV_FROM_TAG(tag))
 	return sparcPciCfgRead32(tag, off);
@@ -210,15 +288,15 @@ sabrePciCfgRead32(PCITAG tag, int off)
 	return (CARD32)(-1);
 
     if (off < 8) {
-	result = (PciReg(pDomain->pci, tag, off, CARD16) << 16) |
-		  PciReg(pDomain->pci, tag, off + 2, CARD16);
+	result = (GetPciWord(pDomain, tag, off) << 16) |
+		  GetPciWord(pDomain, tag, off + 2);
 	return PCI_CPU(result);
     }
 
-    result = (PciReg(pDomain->pci, tag, off + 3, CARD8) << 24) |
-	     (PciReg(pDomain->pci, tag, off + 2, CARD8) << 16) |
-	     (PciReg(pDomain->pci, tag, off + 1, CARD8) <<  8) |
-	     (PciReg(pDomain->pci, tag, off    , CARD8)      );
+    result = (GetPciByte(pDomain, tag, off + 3) << 24) |
+	     (GetPciByte(pDomain, tag, off + 2) << 16) |
+	     (GetPciByte(pDomain, tag, off + 1) <<  8) |
+	     (GetPciByte(pDomain, tag, off    )      );
     return result;
 }
 
@@ -239,13 +317,13 @@ sabrePciCfgWrite32(PCITAG tag, int off, CARD32 val)
 	     (bus == pDomain->bus_min)) {
 	if (off < 8) {
 	    val = PCI_CPU(val);
-	    PciReg(pDomain->pci, tag, off    , CARD16) = val >> 16;
-	    PciReg(pDomain->pci, tag, off + 2, CARD16) = val;
+	    PutPciWord(pDomain, tag, off    , val >> 16);
+	    PutPciWord(pDomain, tag, off + 2, val);
 	} else {
-	    PciReg(pDomain->pci, tag, off    , CARD8) = val;
-	    PciReg(pDomain->pci, tag, off + 1, CARD8) = val >> 8;
-	    PciReg(pDomain->pci, tag, off + 2, CARD8) = val >> 16;
-	    PciReg(pDomain->pci, tag, off + 3, CARD8) = val >> 24;
+	    PutPciByte(pDomain, tag, off    , val);
+	    PutPciByte(pDomain, tag, off + 1, val >>  8);
+	    PutPciByte(pDomain, tag, off + 2, val >> 16);
+	    PutPciByte(pDomain, tag, off + 3, val >> 24);
 	}
     }
 }
@@ -270,13 +348,115 @@ static pciBusFuncs_t sabrePCIFunctions =
     pciAddrNOOP
 };
 
-static long pagemask;
+
+static struct {
+    CARD32 tag;
+    int size;
+} *pAddressSizes = NULL;
+static int nAddressSizes = 0;
+
+/*
+ * Scan the PROM device tree rooted at 'node', accumulating a bit map of PCI
+ * devices that exist, and retrieving base size information.
+ */
+
+static void
+sparcScanPciTree(sparcDomainPtr pDomain, int node)
+{
+    char *prop_val;
+    int prop_len;
+
+    for (node = promGetChild(node);  node;  node = promGetSibling(node)) {
+	/* Retrieve and validate "reg" property */
+	prop_val = promGetProperty("reg", &prop_len);
+	if (prop_val && !(prop_len % 20)) {
+	    /*
+	     * It's unnecessary to scan the entire "reg" property, but I'll do
+	     * so anyway.
+	     */
+	    prop_len /= 20;
+	    for (;  prop_len--;  prop_val += 20)
+		SetBitInMap(PCI_BDF_FROM_TAG(*(CARD32 *)prop_val),
+		    pDomain->bdf_mask);
+	}
+
+	/* Retrieve and validate "assigned-addresses" property */
+	prop_val = promGetProperty("assigned-addresses", &prop_len);
+	if (prop_val && !(prop_len % 20)) {
+	    prop_len /= 20;
+	    for (;  prop_len--;  prop_val += 20) {
+		if (((unsigned char)prop_val[1] < pDomain->bus_min) ||
+		    ((unsigned char)prop_val[1] > pDomain->bus_max) ||
+		    (prop_val[3] & 0x3) || (prop_val[3] < PCI_MAP_REG_START) ||
+		    ((CARD32 *)prop_val)[1] || ((CARD32 *)prop_val)[3] ||
+		    (((CARD32 *)prop_val)[4] < 4) ||
+		    (((CARD32 *)prop_val)[4] & (((CARD32 *)prop_val)[4] - 1)) ||
+		    (((CARD32 *)prop_val)[2] & (((CARD32 *)prop_val)[4] - 1)))
+		    continue;
+
+		if ((prop_val[3] >= PCI_MAP_REG_END) &&
+		    (prop_val[3] != PCI_MAP_ROM_REG) &&
+		    (prop_val[3] != PCI_PCI_BRIDGE_ROM_REG))
+		    continue;
+
+		prop_val[0] = pciNumDomains;
+		pAddressSizes = xnfrealloc(pAddressSizes,
+		    sizeof(*pAddressSizes) * (nAddressSizes + 1));
+		pAddressSizes[nAddressSizes].tag = ((CARD32 *)prop_val)[0];
+		for (pAddressSizes[nAddressSizes].size = -1;
+		     ((CARD32 *)prop_val)[4];
+		     ((CARD32 *)prop_val)[4] >>= 1)
+		    pAddressSizes[nAddressSizes].size++;
+		nAddressSizes++;
+	    }
+	}
+
+	/* Retrieve and validate "class-code" property */
+	prop_val = promGetProperty("class-code", &prop_len);
+	if (prop_val && (prop_len == 4) &&
+	    (prop_val[0] == 0) && (prop_val[1] == PCI_CLASS_BRIDGE) &&
+	    ((prop_val[2] == PCI_SUBCLASS_BRIDGE_PCI) ||
+	     (prop_val[2] == PCI_SUBCLASS_BRIDGE_CARDBUS)))
+		sparcScanPciTree(pDomain, node);
+    }
+}
+
+/* Return the PCI allocation sizes derived above */
+Bool
+xf86GetPciSizeFromOS(PCITAG tag, int Index, int *bits)
+{
+    int i;
+
+    if ((Index < 0) || (Index > 7) || !pAddressSizes || (tag != (CARD32)tag))
+	return FALSE;
+
+    switch (Index) {
+    case 6:
+	tag |= PCI_MAP_ROM_REG;
+	break;
+    case 7:
+	tag |= PCI_PCI_BRIDGE_ROM_REG;
+	break;
+    default:
+	tag |= PCI_MAP_REG_START + (Index * 4);
+	break;
+    }
+
+    for (i = 0;  i < nAddressSizes;  i++) {
+	if (tag == pAddressSizes[i].tag) {
+	    *bits = pAddressSizes[i].size;
+	    return TRUE;
+	}
+    }
+
+    return FALSE;
+}
 
 /* Scan PROM for all PCI host bridges in the system */
 void
 sparcPciInit(void)
 {
-    int node, node2;
+    int node;
 
     if (!xf86LinearVidMem())
 	return;
@@ -294,17 +474,20 @@ sparcPciInit(void)
     for (node = promGetChild(promRootNode);
 	 node;
 	 node = promGetSibling(node)) {
-	unsigned long long pci_addr;
+	unsigned long long pci_addr, pci_size, phys_addr, phys_size;
 	sparcDomainRec     domain;
 	sparcDomainPtr     pDomain;
 	pciBusFuncs_p      pFunctions;
-	char               *prop_val, *bridge_name;
+	char               *prop_val;
 	int                prop_len, bus;
+	char               shared_pci, pciex;
 
 	prop_val = promGetProperty("name", &prop_len);
-	/* Some PROMs include the trailing null;  some don't */
-	if (!prop_val || (prop_len < 3) || (prop_len > 4) ||
-	    strcmp(prop_val, "pci"))
+	if (!prop_val || (prop_len < 3))
+	    continue;
+
+	prop_val[prop_len] = '\0';
+	if (strcmp(prop_val, "pci") && strcmp(prop_val, "pciex"))
 	    continue;
 
 	prop_val = promGetProperty("model", &prop_len);
@@ -319,218 +502,154 @@ sparcPciInit(void)
 	if (!strlen(prop_val))
 	    continue;
 
-	pFunctions = &sparcPCIFunctions;
-	(void)memset(&domain, 0, sizeof(domain));
-
-	do {
-	    static Bool sabre_seen = FALSE;
-
-	    if (!strcmp("SUNW,sabre",   prop_val) ||
-		!strcmp("pci108e,a000", prop_val))
-		bridge_name = "Sabre";
-	    else
-	    if (!strcmp("pci108e,a001", prop_val))
-		bridge_name = "Hummingbird";
-	    else
-		break;
-
-	    /*
-	     * There can only be one "Sabre" bridge in a system.  It provides
-	     * PCI configuration space, a 24-bit I/O space and a 32-bit memory
-	     * space, all three of which are at fixed physical CPU addresses.
-	     */
-	    xf86Msg(X_INFO, "%s PCI host bridge found (\"%s\")\n",
-		bridge_name, prop_val);
-
-	    /* There can only be one Sabre */
-	    if (sabre_seen)
-		goto nextNode;
-	    sabre_seen = TRUE;
-
-	    /* Get "bus-range" property */
-	    prop_val = promGetProperty("bus-range", &prop_len);
-	    if (!prop_val || (prop_len != 8) ||
-		(((unsigned int *)prop_val)[0]) ||
-		(((unsigned int *)prop_val)[1] >= 256))
-		goto nextNode;
-
-	    pci_addr         = 0x01fe01000000ull;
-	    domain.io_addr   = 0x01fe02000000ull;
-	    domain.io_size   = 0x000001000000ull;
-	    domain.mem_addr  = 0x01ff00000000ull;
-	    domain.mem_size  = 0x000100000000ull;
-	    domain.bus_min   = 0;			/* Always */
-	    domain.bus_max   = ((int *)prop_val)[1];
-
+	if (strcmp("SUNW,sabre",   prop_val) &&
+	    strcmp("pci108e,a000", prop_val) &&
+	    strcmp("pci108e,a001", prop_val))
+	    pFunctions = &sparcPCIFunctions;
+	else
 	    pFunctions = &sabrePCIFunctions;
 
-	    goto newDomain;
-	} while (0);
+	if (strcmp("SUNW,psycho", prop_val) &&
+	    strcmp("pci108e,8000", prop_val))
+	    shared_pci = 0;
+	else
+	    shared_pci = 1;
 
-	if (!strcmp("SUNW,psycho",  prop_val) ||
-	    !strcmp("pci108e,8000", prop_val)) {
-	    bridge_name = "Psycho";
+	xf86Msg(X_INFO, "PCI host bridge found (\"%s\")\n", prop_val);
 
-	    /*
-	     * A "Psycho" host bridge provides two PCI interfaces, each with
-	     * its own 16-bit I/O and 31-bit memory spaces.  Both share the
-	     * same PCI configuration space.  Here, they are assigned separate
-	     * domain numbers to prevent unintentional I/O and/or memory
-	     * resource conflicts.
-	     */
-	    xf86Msg(X_INFO,
-		"Psycho PCI host bridge found (\"%s\")\n", prop_val);
+	prop_val = promGetProperty("device_type", &prop_len);
+	if (prop_val && !strcmp("pciex", prop_val))
+	    pciex = 1;
+	else
+	    pciex = 0;
 
-	    /* Get "bus-range" property */
-	    prop_val = promGetProperty("bus-range", &prop_len);
-	    if (!prop_val || (prop_len != 8) ||
-		(((unsigned int *)prop_val)[1] >= 256) ||
-		(((unsigned int *)prop_val)[0] > ((unsigned int *)prop_val)[1]))
+	/* Get "bus-range" property */
+	prop_val = promGetProperty("bus-range", &prop_len);
+	if (!prop_val || (prop_len != 8) ||
+	    (((unsigned int *)prop_val)[1] >= 256) ||
+	    (((unsigned int *)prop_val)[0] > ((unsigned int *)prop_val)[1]))
+	    continue;
+
+	(void)memset(&domain, 0, sizeof(domain));
+
+	domain.bus_min = ((int *)prop_val)[0];
+	domain.bus_max = ((int *)prop_val)[1];
+
+	/* Retrieve and validate "ranges" property */
+	prop_val = promGetProperty("ranges", &prop_len);
+	if (!prop_val || (prop_len <= 0) || (prop_len % 28))
+	    continue;
+
+	/* Extract address space location and size from "ranges" property */
+	pci_addr = pci_size = 0;
+	for (prop_len /= 28;  prop_len--;  prop_val += 28) {
+	   /* Address space size must be a power of two */
+	   phys_size =
+		((unsigned long long)((unsigned int *)prop_val)[5] << 32) |
+		(unsigned long long)((unsigned int *)prop_val)[6];
+	   if (!phys_size || (phys_size & (phys_size - 1)))
 		goto nextNode;
 
-	    domain.bus_min = ((int *)prop_val)[0];
-	    domain.bus_max = ((int *)prop_val)[1];
-
-	    /* Get "ranges" property */
-	    prop_val = promGetProperty("ranges", &prop_len);
-	    if (!prop_val || (prop_len != 112) ||
-		prop_val[0] || (prop_val[28] != 0x01u) ||
-		(prop_val[56] != 0x02u) || (prop_val[84] != 0x03u) ||
-		(((unsigned int *)prop_val)[4] != 0x01000000u) ||
-		((unsigned int *)prop_val)[5] ||
-		((unsigned int *)prop_val)[12] ||
-		(((unsigned int *)prop_val)[13] != 0x00010000u) ||
-		((unsigned int *)prop_val)[19] ||
-		(((unsigned int *)prop_val)[20] != 0x80000000u) ||
-		((((unsigned int *)prop_val)[11] & ~0x00010000u) !=
-		 0x02000000u) ||
-		(((unsigned int *)prop_val)[18] & ~0x80000000u) ||
-		(((unsigned int *)prop_val)[3] !=
-		 ((unsigned int *)prop_val)[10]) ||
-		(((unsigned int *)prop_val)[17] !=
-		 ((unsigned int *)prop_val)[24]) ||
-		(((unsigned int *)prop_val)[18] !=
-		 ((unsigned int *)prop_val)[25]) ||
-		(((unsigned int *)prop_val)[19] !=
-		 ((unsigned int *)prop_val)[26]) ||
-		(((unsigned int *)prop_val)[20] !=
-		 ((unsigned int *)prop_val)[27]))
+	   /* Address space must be size-aligned */
+	   phys_addr =
+		((unsigned long long)((unsigned int *)prop_val)[3] << 32) |
+		(unsigned long long)((unsigned int *)prop_val)[4];
+	   if (phys_addr % phys_size)
 		goto nextNode;
 
-	    /* Use memcpy() to avoid alignment issues */
-	    (void)memcpy(&pci_addr, prop_val + 12,
-			 sizeof(pci_addr));
-	    (void)memcpy(&domain.io_addr, prop_val + 40,
-			 sizeof(domain.io_addr));
-	    (void)memcpy(&domain.mem_addr, prop_val + 68,
-			 sizeof(domain.mem_addr));
+	   switch (prop_val[0] & 0x03) {
+	   case 0:	/* PCI configuration space */
+		if ((pci_addr == phys_addr) && (pci_size == phys_size))
+		    break;
 
-	    domain.io_size  = 0x000000010000ull;
-	    domain.mem_size = 0x000080000000ull;
-
-	    goto newDomain;
-	}
-
-	do {
-	    volatile unsigned long long mem_match, mem_mask, io_match, io_mask;
-	    unsigned long Offset;
-	    pointer pSchizo;
-
-	    if (!strcmp("SUNW,schizo",    prop_val) ||
-		!strcmp("pci108e,8001",   prop_val))
-		bridge_name = "Schizo";
-	    else
-	    if (!strcmp("SUNW,schizo+",   prop_val) ||
-		!strcmp("pci108e,8002",   prop_val))
-		bridge_name = "Schizo+";
-	    else
-	    if (!strcmp("SUNW,tomatillo", prop_val) ||
-		!strcmp("pci108e,a801",   prop_val))
-		bridge_name = "Tomatillo";
-	    else
+		if (pci_size)
+		    goto nextNode;
+		if (shared_pci) {
+		    phys_size <<= 1;
+		    if (!phys_size)
+			goto nextNode;
+		}
+		pci_addr = phys_addr;
+		pci_size = phys_size;
 		break;
 
-	    /*
-	     * I have no docs on the "Schizo", nor on any of its revisions, but
-	     * judging from the Linux kernel, it also provides two PCI domains.
-	     * Each PCI configuration space is the usual 16M in size, followed
-	     * by a variable-length I/O space.  Each domain also provides a
-	     * variable-length memory space.  The kernel seems to think the I/O
-	     * spaces are 16M long, and the memory spaces, 2G, but these
-	     * assumptions are actually only present in source code comments.
-	     * Sun has, however, confirmed to me the validity of these
-	     * assumptions.
-	     */
-	    xf86Msg(X_INFO, "%s PCI host bridge found (\"%s\")\n",
-		bridge_name, prop_val);
+	   case 1:	/* I/O space */
+		if ((domain.io_addr == phys_addr) &&
+		    (domain.io_size == phys_size))
+		    break;
 
-	    /* Get "bus-range" property */
-	    prop_val = promGetProperty("bus-range", &prop_len);
-	    if (!prop_val || (prop_len != 8) ||
-		(((unsigned int *)prop_val)[1] >= 256) ||
-		(((unsigned int *)prop_val)[0] > ((unsigned int *)prop_val)[1]))
-		goto nextNode;
+		if (domain.io_size)
+		    goto nextNode;
+		domain.io_addr = phys_addr;
+		domain.io_size = phys_size;
+		break;
 
-	    domain.bus_min = ((int *)prop_val)[0];
-	    domain.bus_max = ((int *)prop_val)[1];
+	   case 2:	/* 32-bit memory space */
+		if ((domain.mem_addr == phys_addr) &&
+		    (domain.mem_size == phys_size))
+		    break;
 
-	    /* Get "reg" property */
-	    prop_val = promGetProperty("reg", &prop_len);
-	    if (!prop_val || ((prop_len != 48) && (prop_len != 64)))
-		goto nextNode;
+		if (domain.mem_size)
+		    goto nextNode;
+		domain.mem_addr = phys_addr;
+		domain.mem_size = phys_size;
+		break;
 
-	    /* Temporarily map some of Schizo's registers */
-	    pSchizo = sparcMapAperture(-1, VIDMEM_MMIO,
-		((unsigned long long *)prop_val)[2] - 0x000000010000ull,
-		0x00010000ul);
+	   case 3:	/* 64-bit memory space */
+	   default:	/* Muffle compiler */
+		break;	/* Ignore, for now */
+	   }
+	}
 
-	    /* Determine where PCI config, I/O and memory spaces reside */
-	    if ((((unsigned long long *)prop_val)[0] & 0x000000700000ull) ==
-		0x000000600000ull)
-		Offset = 0x0040;
-	    else
-		Offset = 0x0060;
+	/* Enforce minimum address space sizes */
+	if (!domain.mem_size ||
+	    (domain.io_size < 0x00010000ull) ||
+	    (pci_size < 0x01000000ull))
+	    continue;
 
-	    mem_match = PciReg(pSchizo, 0, Offset, unsigned long long);
-	    mem_mask  = PciReg(pSchizo, 0, Offset + 8, unsigned long long);
-	    io_match  = PciReg(pSchizo, 0, Offset + 16, unsigned long long);
-	    io_mask   = PciReg(pSchizo, 0, Offset + 24, unsigned long long);
-
-	    /* Unmap Schizo registers */
-	    xf86UnMapVidMem(-1, pSchizo, 0x00010000ul);
-
-	    /* Calculate sizes */
-	    mem_mask = (((mem_mask - 1) ^ mem_mask) >> 1) + 1;
-	    io_mask  = (((io_mask  - 1) ^ io_mask ) >> 1) + 1;
-
-	    if (io_mask <= 0x000001000000ull)	/* Nothing left for I/O */
-		goto nextNode;
-
-	    domain.mem_addr = mem_match & ~0x8000000000000000ull;
-	    domain.mem_size = mem_mask;
-	    pci_addr        = io_match  & ~0x8000000000000000ull;
-	    domain.io_addr  = pci_addr  +  0x0000000001000000ull;
-	    domain.io_size  = io_mask   -  0x0000000001000000ull;
-
-	    goto newDomain;
-	} while (0);
-
-	xf86Msg(X_WARNING, "Unknown PCI host bridge: \"%s\"\n", prop_val);
-	continue;
-
-newDomain:
 	if (pciNumDomains >= MAX_DOMAINS) {
 	    xf86Msg(X_ERROR, "MAX_PCI_BUSES needs to be increased to"
-		    " accomodate this %s host bridge\n", bridge_name);
+		    " accomodate this host bridge\n");
 	    pciNumDomains++;
 	    continue;
 	}
 
-	/* Only map as much PCI configuration as we need */
-	domain.pci = (char *)sparcMapAperture(-1, VIDMEM_MMIO,
-	    pci_addr + PCI_MAKE_TAG(domain.bus_min, 0, 0),
-	    PCI_MAKE_TAG(domain.bus_max - domain.bus_min + 1, 0, 0)) -
-	    PCI_MAKE_TAG(domain.bus_min, 0, 0);
+	domain.tagMultiplier = pci_size >> 24;
+	domain.maxOffset = domain.tagMultiplier << 8;
+
+	/* Only mmap as much PCI configuration as we need */
+	phys_addr = pci_addr +
+	    (PCI_MAKE_TAG(domain.bus_min, 0, 0) * domain.tagMultiplier);
+	phys_addr &= ~pagemask;
+	phys_size = pci_addr +
+	    (PCI_MAKE_TAG(domain.bus_max + 1, 0, 0) * domain.tagMultiplier);
+	phys_size += pagemask;
+	phys_size &= ~pagemask;
+	phys_size -= phys_addr;
+
+	domain.pci = sparcMapAperture(-1, VIDMEM_MMIO, phys_addr, phys_size);
+	if (domain.pci == MAP_FAILED)
+	    continue;
+
+	domain.pci = (char *)domain.pci - (phys_addr - pci_addr);
+
+	/* Allocate bus/device/function bit map */
+	domain.bdf_mask = (unsigned char *)xnfcalloc(1,
+	    (domain.bus_max - domain.bus_min + 1) << 5) -
+	    (domain.bus_min << 5);
+
+	xf86MsgVerb(X_INFO, 4, "Adding PCI domain %d:\n", pciNumDomains);
+	xf86MsgVerb(X_INFO, 4,
+	    "PCI Configuration space: 0x%016llx, size: 0x%09llx\n",
+	    pci_addr, pci_size);
+	xf86MsgVerb(X_INFO, 4,
+	    "PCI Input/Output space:  0x%016llx, size: 0x%09llx\n",
+	    domain.io_addr, domain.io_size);
+	xf86MsgVerb(X_INFO, 4,
+	    "PCI Memory space:        0x%016llx, size: 0x%09llx\n",
+	    domain.mem_addr, domain.mem_size);
+	xf86MsgVerb(X_INFO, 4, "PCI Bus range:           %d-%d\n",
+	    domain.bus_min, domain.bus_max);
 
 	/* Allocate a domain record */
 	pDomain = xnfalloc(sizeof(sparcDomainRec));
@@ -550,6 +669,7 @@ newDomain:
 	pciBusInfo[bus]->numDevices = 32;
 	pciBusInfo[bus]->funcs = pFunctions;
 	pciBusInfo[bus]->pciBusPriv = pDomain;
+	pciBusInfo[bus]->pciMaxOffset = domain.maxOffset;
 	while (++bus < pciNumBuses) {
 	    pciBusInfo[bus] = xnfalloc(sizeof(pciBusInfo_t));
 	    *(pciBusInfo[bus]) = *(pciBusInfo[bus - 1]);
@@ -557,7 +677,7 @@ newDomain:
 	}
 
 	/* Next domain, please... */
-	xf86DomainInfo[pciNumDomains++] = pDomain;
+	xf86DomainInfo[pciNumDomains] = pDomain;
 
 	/*
 	 * OK, enough of the straight-forward stuff.  Time to deal with some
@@ -583,9 +703,9 @@ newDomain:
 	 * detect when master aborts occur.  Obviously, PCI discovery is much
 	 * simpler when master aborts are allowed to complete normally.
 	 *
-	 * Unfortunately, a number of non-Intel PCI implementations have chosen
-	 * to treat master aborts as severe errors.  The net effect is to
-	 * cripple PCI discovery algorithms in userland.
+	 * Unfortunately, a number of PCI implementations have chosen to treat
+	 * master aborts as severe errors.  The net effect is to cripple PCI
+	 * discovery algorithms in userland.
 	 *
 	 * On SPARCs, master aborts cause a number of different behaviours,
 	 * including delivering a signal to the userland application, rebooting
@@ -621,27 +741,32 @@ newDomain:
 	 * VGA, or if a PCI device actually implements PCI disablement.
 	 *
 	 * ---  TSI @ UQV  2001.09.19
+	 *
+	 * This has been changed to generate a bit map for all of an
+	 * interface's buses, because it appears that probing for a PCI Express
+	 * switch's downstream ports cannot be done without generating
+	 * interrupts that are untrappable from userland.  In effect, PCI
+	 * Express has entrenched the hard-failing of unsupported requests
+	 * (which include its equivalent to master aborts).
+	 *
+	 * ---  TSI @ UQV  2008.01.31
 	 */
-	for (node2 = promGetChild(node);
-	     node2;
-	     node2 = promGetSibling(node2)) {
-	    /* Get "reg" property */
-	    prop_val = promGetProperty("reg", &prop_len);
-	    if (!prop_val || (prop_len % 20))
-		continue;
+	sparcScanPciTree(&domain, node);
 
-	    /*
-	     * It's unnecessary to scan the entire "reg" property, but I'll do
-	     * so anyway.
-	     */
-	    prop_len /= 20;
-	    for (;  prop_len--;  prop_val += 20)
-		SetBitInMap(PCI_DFN_FROM_TAG(*(PCITAG *)prop_val),
-		    pDomain->dfn_mask);
-	}
+	/*
+	 * Assume the host bridge is device 0, function 0 on its bus.  Note
+	 * that this is valid for PCI Express as well, because the spec
+	 * requires that the other end of the link from a root port, or any
+	 * of a switch's downstream ports, be device 0, function 0.
+	 */
+	SetBitInMap(domain.bus_min << 8, pDomain->bdf_mask);
 
-	/* Assume the host bridge is device 0, function 0 on its bus */
-	SetBitInMap(0, pDomain->dfn_mask);
+	if ((domain.bus_min < domain.bus_max) &&
+	    (!pciex || (xf86Info.estimateSizesAggressively > 0)))
+	    memset(domain.bdf_mask + ((domain.bus_min + 1) << 5), 0xff,
+		(domain.bus_max - domain.bus_min) << 5);
+
+	pciNumDomains++;
 
 nextNode:;
     }
@@ -680,6 +805,9 @@ xf86MapDomainMemory(int ScreenNum, int Flags, PCITAG Tag,
 
     result = sparcMapAperture(ScreenNum, Flags, pDomain->mem_addr + Base, Size);
 
+    if (result == MAP_FAILED)
+	FatalError("xf86MapDomainMemory():  sparcMapAperture() failure.\n");
+
     if (apertureFd >= 0) {
 	close(apertureFd);
 	apertureFd = -1;
@@ -705,6 +833,9 @@ xf86MapDomainIO(int ScreenNum, int Flags, PCITAG Tag,
 	pDomain->io = sparcMapAperture(ScreenNum, Flags,
 	    pDomain->io_addr, pDomain->io_size);
 
+	if (pDomain->io == MAP_FAILED)
+	    FatalError("xf86MapDomainIO():  sparcMapAperture() failure.\n");
+
 	if (apertureFd >= 0) {
 	    close(apertureFd);
 	    apertureFd = -1;
@@ -868,13 +999,16 @@ xf86LocatePciMemoryArea(PCITAG Tag, char **devName, unsigned int *devOffset,
     return TRUE;
 }
 
-#endif /* !INCLUDE_XF86_NO_DOMAIN */
+#endif /* !defined(INCLUDE_XF86_NO_DOMAIN) */
 
-#endif /* defined(sun) */
+#endif /* defined(sun) || defined(__OpenBSD__) */
 
 #if defined(ARCH_PCI_PCI_BRIDGE)
 
+#define APB_DEVICE_ID		(DEVID(VENDOR_SUN, CHIP_SIMBA))
+
 /* Definitions specific to Sun's APB P2P bridge (a.k.a. Simba) */
+#define APB_INT_ACK		0xB8
 #define APB_IO_ADDRESS_MAP	0xDE
 #define APB_MEM_ADDRESS_MAP	0xDF
 
@@ -929,7 +1063,7 @@ simbaCheckBus(CARD16 pcicommand, int bus)
 		(pPCI->pci_vendor == PCI_VENDOR_CREATIVE) ||
 		(pPCI->pci_vendor == PCI_VENDOR_ENSONIQ))
 		break;
-            /* Fall through */
+	    /* Fall through */
 
 	case PCI_CLASS_DISPLAY:
 	    continue;
@@ -964,13 +1098,29 @@ simbaCheckBus(CARD16 pcicommand, int bus)
     }
 }
 
+static CARD32 (*wrappedPciReadLong)(PCITAG, int) = NULL;
+
+/*
+ * Wrapper function to avoid the interrupt acknowledge cycle that is generated
+ * on a Simba's secondary bus when its INT_ACK register is read.
+ */
+static CARD32
+simbaPciCfgRead32(PCITAG tag, int offset)
+{
+    if ((offset != APB_INT_ACK) ||
+	((*wrappedPciReadLong)(tag, PCI_ID_REG) != APB_DEVICE_ID))
+	return (*wrappedPciReadLong)(tag, offset);
+
+    return PCI_NOT_FOUND;
+}
+
 static pciConfigPtr
 simbaVerifyBus(int bus)
 {
     pciConfigPtr pPCI;
     if ((bus < 0) || (bus >= pciNumBuses) ||
 	!pciBusInfo[bus] || !(pPCI = pciBusInfo[bus]->bridge) ||
-	(pPCI->pci_device_vendor != DEVID(VENDOR_SUN, CHIP_SIMBA)))
+	(pPCI->pci_device_vendor != APB_DEVICE_ID))
 	return NULL;
 
     return pPCI;
@@ -1146,16 +1296,22 @@ void ARCH_PCI_PCI_BRIDGE(pciConfigPtr pPCI)
     pciBusInfo_t *pBusInfo;
     CARD16 pcicommand;
 
-    if (pPCI->pci_device_vendor != DEVID(VENDOR_SUN, CHIP_SIMBA))
+    if (pPCI->pci_device_vendor != APB_DEVICE_ID)
 	return;
 
     pBusInfo = pPCI->businfo;
 
-    simbaBusFuncs = *(pBusInfo->funcs);
-    simbaBusFuncs.pciControlBridge = simbaControlBridge;
-    simbaBusFuncs.pciGetBridgeResources = simbaGetBridgeResources;
+    if (pBusInfo->funcs != &simbaBusFuncs) {
+	simbaBusFuncs = *(pBusInfo->funcs);
 
-    pBusInfo->funcs = &simbaBusFuncs;
+	wrappedPciReadLong = simbaBusFuncs.pciReadLong;
+	simbaBusFuncs.pciReadLong = simbaPciCfgRead32;
+
+	simbaBusFuncs.pciControlBridge = simbaControlBridge;
+	simbaBusFuncs.pciGetBridgeResources = simbaGetBridgeResources;
+
+	pBusInfo->funcs = &simbaBusFuncs;
+    }
 
     if (!simbavgaRoutingAllow)
 	return;
