@@ -1,4 +1,4 @@
-/* $Xorg: Handlers.c,v 1.3 2000/08/17 19:53:28 cpqbld Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xnest/Handlers.c,v 1.4 2005/10/14 15:17:14 tsi Exp $ */
 /*
 
 Copyright 1993 by Davor Matic
@@ -12,10 +12,9 @@ the suitability of this software for any purpose.  It is provided "as
 is" without express or implied warranty.
 
 */
-/* $XFree86: xc/programs/Xserver/hw/xnest/Handlers.c,v 1.3 2003/11/16 05:05:20 dawes Exp $ */
 
-#include "X.h"
-#include "Xproto.h"
+#include <X11/X.h>
+#include <X11/Xproto.h>
 #include "screenint.h"
 #include "input.h"
 #include "misc.h"
