@@ -1,3 +1,4 @@
+/* $XFree86: xc/programs/Xserver/os/log.c,v 1.18 2008/05/01 16:13:24 tsi Exp $ */
 /*
 
 Copyright 1987, 1998  The Open Group
@@ -50,7 +51,7 @@ OR PERFORMANCE OF THIS SOFTWARE.
 */
 
 /*
- * Copyright (c) 1997-2006 by The XFree86 Project, Inc.
+ * Copyright (c) 1997-2008 by The XFree86 Project, Inc.
  * All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining
@@ -96,8 +97,6 @@ OR PERFORMANCE OF THIS SOFTWARE.
  * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-/* $XFree86: xc/programs/Xserver/os/log.c,v 1.17 2006/11/03 21:43:24 tsi Exp $ */
-
 #include <X11/Xos.h>
 #include <stdio.h>
 #include <time.h>
@@ -117,6 +116,8 @@ static Bool logFlush = FALSE;
 static Bool logSync = FALSE;
 static int logVerbosity = DEFAULT_LOG_VERBOSITY;
 static int logFileVerbosity = DEFAULT_LOG_FILE_VERBOSITY;
+static char *logFileName = NULL;
+
 
 /* Buffer to information logged before the log file is opened. */
 static char *saveBuffer = NULL;
@@ -167,8 +168,6 @@ static Bool needBuffer = TRUE;
 const char *
 LogInit(const char *fname, const char *backup)
 {
-    char *logFileName = NULL;
-
     if (fname && *fname) {
 	xasprintf(&logFileName, fname, display);
 	if (!logFileName)
@@ -249,6 +248,20 @@ LogSetParameter(LogParameter param, int value)
     case XLOG_FILE_VERBOSITY:
 	logFileVerbosity = value;
 	return TRUE;
+    case XLOG_STDERR:
+	/* Redirect the server's stderr to log */
+	if (!logFileName || (logFile == stderr))
+	    return FALSE;
+
+	LogWrite(-1, "Redirecting server stderr to \"%s\"\n", logFileName);
+	fclose(logFile);
+	logFile = freopen(logFileName, "a+", stderr);
+	if (!logFile)
+	    abort();
+	if (logVerbosity < logFileVerbosity)
+	    logVerbosity = logFileVerbosity;
+	logFileVerbosity = -2;
+	return TRUE;
     default:
 	return FALSE;
     }
@@ -260,23 +273,32 @@ void
 LogVWrite(int verb, const char *f, va_list args)
 {
     static char tmpBuffer[1024];
-    int len = 0;
+    int len;
+
+    if ((verb > logFileVerbosity) && (verb > logVerbosity))
+	return;
 
     /*
      * Since a va_list can only be processed once, write the string to a
      * buffer, and then write the buffer out to the appropriate output
      * stream(s).
      */
-    if (verb < 0 || logFileVerbosity >= verb || logVerbosity >= verb) {
-	vsnprintf(tmpBuffer, sizeof(tmpBuffer), f, args);
-	len = strlen(tmpBuffer);
-    }
-    if ((verb < 0 || logVerbosity >= verb) && len > 0)
+    vsnprintf(tmpBuffer, sizeof(tmpBuffer), f, args);
+    len = strlen(tmpBuffer);
+    if (len <= 0)
+	return;
+
+    if (verb <= logVerbosity) {
 	fwrite(tmpBuffer, len, 1, stderr);
-    if ((verb < 0 || logFileVerbosity >= verb) && len > 0) {
+	if (logFile == stderr)
+	    goto FlushLog;
+    }
+
+    if (verb <= logFileVerbosity) {
 	if (logFile) {
 	    fwrite(tmpBuffer, len, 1, logFile);
 	    if (logFlush || logSync) {
+	FlushLog:
 		fflush(logFile);
 		if (logSync)
 		    fsync(fileno(logFile));
